SPOOL c:\sql\creartaules.log
/

-- DROPS --
DROP TABLE localitat CASCADE CONSTRAINTS;
/
DROP SEQUENCE seq_LOCALITAT;
/
DROP TABLE poblacio CASCADE CONSTRAINTS;
/
DROP SEQUENCE seq_POBLACIO;
/
DROP TABLE pais CASCADE CONSTRAINTS;
/
DROP SEQUENCE seq_PAIS;
/
DROP TABLE fabrica CASCADE CONSTRAINTS;
/
DROP TABLE model_comptador CASCADE CONSTRAINTS;
/
DROP SEQUENCE seq_MODEL_COMPTADOR;
/
DROP TABLE modalitat_lectura CASCADE CONSTRAINTS;
/
DROP SEQUENCE seq_MODALITAT_LECTURA;
/
DROP TABLE tipus_centralprod CASCADE CONSTRAINTS;
/
DROP SEQUENCE seq_TIPUS_CENTRALPROD;
/
DROP TABLE linia_comunicacio CASCADE CONSTRAINTS;
/
DROP SEQUENCE seq_LINIA_COMUNICACIO;
/
DROP TABLE central_produccio CASCADE CONSTRAINTS;
/
DROP SEQUENCE seq_CENTRAL_PRODUCCIO;
/
DROP TABLE hist_central_prod CASCADE CONSTRAINTS;
/
DROP TABLE central_distribucio CASCADE CONSTRAINTS;
/
DROP SEQUENCE seq_CENTRAL_DISTRIBUCIO;
/
DROP TABLE distribucio_comptador CASCADE CONSTRAINTS;
/
DROP TABLE client CASCADE CONSTRAINTS;
/
DROP TABLE comptador CASCADE CONSTRAINTS;
/
DROP SEQUENCE seq_COMPTADOR;
/
DROP TABLE lectura_comptador CASCADE CONSTRAINTS;
/
DROP TABLE est_consumcomptprod CASCADE CONSTRAINTS;
/
DROP TABLE est_consumliniaany CASCADE CONSTRAINTS;
/
DROP TABLE est_liniamescarregada CASCADE CONSTRAINTS;
/
DROP TABLE est_anyliniesenconsum CASCADE CONSTRAINTS;
/
DROP TABLE est_consumcpany CASCADE CONSTRAINTS;
/
DROP TABLE est_sumacpany CASCADE CONSTRAINTS;
/
DROP TABLE est_top10comptadors CASCADE CONSTRAINTS;
/
DROP TABLE est_consummigclients CASCADE CONSTRAINTS;
/
DROP TABLE logs CASCADE CONSTRAINTS;
/
DROP SEQUENCE seq_LOGS;
/

-- PAIS --
CREATE TABLE pais(
  idPais NUMBER CONSTRAINT PK_pais PRIMARY KEY, 
  nompais VARCHAR2(50 BYTE) CONSTRAINT NN_nompais NOT NULL
);
/

CREATE SEQUENCE seq_PAIS INCREMENT BY 1 START WITH 1;
/

CREATE OR REPLACE TRIGGER TG_Ins_idPais
  BEFORE INSERT ON PAIS
  FOR EACH ROW
BEGIN
  SELECT seq_PAIS.NEXTVAL INTO :NEW.idPais
  FROM DUAL;
END TG_Ins_idPais;
/

-- POBLACIO --
CREATE TABLE poblacio(
  idPoblacio NUMBER CONSTRAINT PK_poblacio PRIMARY KEY, 
  nompoblacio VARCHAR2(50 BYTE) CONSTRAINT NN_nompoblacio NOT NULL
  CONSTRAINT UN_pob_nompoblacio UNIQUE,
  idpais NUMBER CONSTRAINT NN_idpais NOT NULL 
  CONSTRAINT FK_pais_idpais REFERENCES pais(idpais)
);
/

CREATE SEQUENCE seq_POBLACIO INCREMENT BY 1 START WITH 1;
/

CREATE OR REPLACE TRIGGER TG_Ins_idPobpacio
  BEFORE INSERT ON POBLACIO
  FOR EACH ROW
BEGIN
  SELECT seq_POBLACIO.NEXTVAL INTO :NEW.idPoblacio
  FROM DUAL;
END TG_Ins_idPoblacio;
/

-- LOCALITAT --
CREATE TABLE localitat(
  idLocalitat NUMBER CONSTRAINT PK_localitat PRIMARY KEY, 
  nomlocalitat VARCHAR2(50 BYTE) CONSTRAINT NN_nomlocalitat NOT NULL,
  codpostal NUMBER CONSTRAINT NN_codpostal NOT NULL
  CONSTRAINT UN_loc_codpostal UNIQUE,
  idpoblacio NUMBER CONSTRAINT NN_idpoblacio NOT NULL
  CONSTRAINT FK_poblacio_idpoblacio REFERENCES poblacio(idpoblacio)
);
/

CREATE SEQUENCE seq_LOCALITAT INCREMENT BY 1 START WITH 1;
/

CREATE OR REPLACE TRIGGER TG_Ins_idLocalitat
  BEFORE INSERT ON LOCALITAT
  FOR EACH ROW
BEGIN
  SELECT seq_LOCALITAT.NEXTVAL INTO :NEW.idLocalitat
  FROM DUAL;
END TG_Ins_idLocalitat;
/

-- FABRICA --
CREATE TABLE fabrica(
  CIFfabrica VARCHAR2(10 BYTE) CONSTRAINT PK_fabrica PRIMARY KEY, 
  nomfabrica VARCHAR2(50 BYTE) CONSTRAINT NN_nomfabrica NOT NULL
);
/

-- MODEL COMPTADOR --
CREATE TABLE model_comptador(
  idModel NUMBER CONSTRAINT PK_model_comptador PRIMARY KEY, 
  CIFfabrica VARCHAR2(10 BYTE) CONSTRAINT NN_CIFfabrica NOT NULL
  CONSTRAINT FK_fabrica_CIFfabrica REFERENCES fabrica(CIFfabrica), 
  nommodel VARCHAR2(50 BYTE) CONSTRAINT NN_nommodel NOT NULL,
  any_fabricat DATE
);
/

CREATE SEQUENCE seq_MODEL_COMPTADOR INCREMENT BY 1 START WITH 1;
/

CREATE OR REPLACE TRIGGER TG_Ins_idModel
  BEFORE INSERT ON MODEL_COMPTADOR
  FOR EACH ROW
BEGIN
  SELECT seq_MODEL_COMPTADOR.NEXTVAL INTO :NEW.idModel
  FROM DUAL;
END TG_Ins_idModel;
/

-- MODALITAT_LECTURA --
CREATE TABLE modalitat_lectura(
  idModalitat NUMBER CONSTRAINT PK_modalitat_lectura PRIMARY KEY, 
  nommodalitat VARCHAR2(50 BYTE) CONSTRAINT NN_nommodalitat NOT NULL
);
/

CREATE SEQUENCE seq_MODALITAT_LECTURA INCREMENT BY 1 START WITH 1;
/

CREATE OR REPLACE TRIGGER TG_Ins_idModalitat
  BEFORE INSERT ON MODALITAT_LECTURA
  FOR EACH ROW
BEGIN
  SELECT seq_MODALITAT_LECTURA.NEXTVAL INTO :NEW.idModalitat
  FROM DUAL;
END TG_Ins_idModalitat;
/

-- TIPUS_CENTRALPROD --
CREATE TABLE tipus_centralprod(
  idTipusprod NUMBER CONSTRAINT PK_tipus_centralprod PRIMARY KEY, 
  nomtipus VARCHAR2(50 BYTE) CONSTRAINT NN_nomtipus NOT NULL
  CONSTRAINT UN_tcp_nomtipus UNIQUE
);
/

CREATE SEQUENCE seq_TIPUS_CENTRALPROD INCREMENT BY 1 START WITH 1;
/

CREATE OR REPLACE TRIGGER TG_Ins_idTipusprod
  BEFORE INSERT ON TIPUS_CENTRALPROD
  FOR EACH ROW
BEGIN
  SELECT seq_TIPUS_CENTRALPROD.NEXTVAL INTO :NEW.idTipusprod
  FROM DUAL;
END TG_Ins_idTipusprod;
/

-- CENTRAL_PRODUCCIO --
CREATE TABLE central_produccio(
  idCentralprod NUMBER CONSTRAINT PK_central_produccio PRIMARY KEY, 
  idTipusprod NUMBER CONSTRAINT NN_idTipusprod NOT NULL
  CONSTRAINT FK_tcp_idTipusprod REFERENCES tipus_centralprod(idTipusprod),
  energia_max NUMBER,
  direccio VARCHAR2(100 BYTE) CONSTRAINT NN_CP_direccio NOT NULL,
  idlocalitat NUMBER CONSTRAINT NN_CP_idlocalitat NOT NULL
  CONSTRAINT FK_loc_CP_idlocalitat REFERENCES localitat(idlocalitat),
  data_it DATE,
  atribut_1 NUMBER,
  atribut_2 NUMBER,
  data_alta DATE CONSTRAINT NN_CP_data_alta NOT NULL,
  data_baixa DATE,
  data_last_mod DATE
);
/

CREATE SEQUENCE seq_CENTRAL_PRODUCCIO INCREMENT BY 1 START WITH 1;
/

CREATE OR REPLACE TRIGGER TG_Ins_idCentralprod
  BEFORE INSERT ON CENTRAL_PRODUCCIO
  FOR EACH ROW
BEGIN
  SELECT seq_CENTRAL_PRODUCCIO.NEXTVAL INTO :NEW.idCentralprod
  FROM DUAL;
END TG_Ins_idCentralprod;
/

-- LINIA COMUNICACIO --
CREATE TABLE linia_comunicacio(
  idlinia NUMBER CONSTRAINT PK_linia_comunicacio PRIMARY KEY,
  idCentralprod NUMBER CONSTRAINT NN_LC_idCentralprod NOT NULL
  CONSTRAINT FK_LC_idCentralprod REFERENCES central_produccio(idCentralprod), 
  capacitat_max NUMBER CONSTRAINT NN_capacitat_max NOT NULL,
  data_alta DATE CONSTRAINT NN_LD_data_alta NOT NULL,
  data_baixa DATE,
  data_last_mod DATE 
);
/

CREATE SEQUENCE seq_LINIA_COMUNICACIO INCREMENT BY 1 START WITH 1;

CREATE OR REPLACE TRIGGER TG_Ins_idlinia
  BEFORE INSERT ON LINIA_COMUNICACIO
  FOR EACH ROW
BEGIN
  SELECT seq_LINIA_COMUNICACIO.NEXTVAL INTO :NEW.idlinia
  FROM DUAL;
END TG_Ins_idlinia;
/

-- HIST_CENTRAL_PROD --
CREATE TABLE hist_central_prod(
  idCentralprod NUMBER CONSTRAINT FK_HCP_idCentralprod REFERENCES central_produccio(idCentralprod), 
  data_inici DATE,
  data_fi DATE,
  energia_mes NUMBER CONSTRAINT NN_energia_mes NOT NULL,
  CONSTRAINT PK_hist_central_prod PRIMARY KEY (idCentralprod,data_inici,data_fi) 
);
/

-- CENTRAL_DISTRIBUCIO --
CREATE TABLE central_distribucio(
  idCentraldis NUMBER CONSTRAINT PK_central_distribucio PRIMARY KEY, 
  idCentralprod NUMBER CONSTRAINT NN_idCentralprod NOT NULL
  CONSTRAINT FK_CD_idCentralprod REFERENCES central_produccio(idCentralprod),
  idLinia NUMBER CONSTRAINT NN_CD_idLinia NOT NULL
  CONSTRAINT FK_CD_idLinia REFERENCES linia_comunicacio(idLinia),
  energia_max NUMBER CONSTRAINT NN_CD_energia_max NOT NULL,
  direccio VARCHAR2(100 BYTE) CONSTRAINT NN_CD_direccio NOT NULL,
  idlocalitat NUMBER CONSTRAINT NN_CD_idlocalitat NOT NULL
  CONSTRAINT FK_loc_CD_idlocalitat REFERENCES localitat(idlocalitat),
  data_alta DATE CONSTRAINT NN_CD_data_alta NOT NULL,
  data_baixa DATE,
  data_last_mod DATE
);
/

CREATE SEQUENCE seq_CENTRAL_DISTRIBUCIO INCREMENT BY 1 START WITH 1;
/

CREATE OR REPLACE TRIGGER TG_Ins_idCentraldis
  BEFORE INSERT ON CENTRAL_DISTRIBUCIO
  FOR EACH ROW
BEGIN
  SELECT seq_CENTRAL_DISTRIBUCIO.NEXTVAL INTO :NEW.idCentraldis
  FROM DUAL;
END TG_Ins_idCentraldis;
/

-- CLIENT --
CREATE TABLE client(
  NIF VARCHAR2 (10 BYTE) CONSTRAINT PK_client PRIMARY KEY, 
  tipus_client NUMBER CONSTRAINT NN_tipus_client NOT NULL
  CONSTRAINT CH_tipus_client CHECK  (tipus_client in (1,2)),
  nom VARCHAR2(40 BYTE) CONSTRAINT NN_CL_nom NOT NULL,
  cognom1 VARCHAR2(50 BYTE),
  cognom2 VARCHAR2(50 BYTE),
  data_naixement DATE,
  direccio VARCHAR2(100 BYTE) CONSTRAINT NN_CL_direccio NOT NULL,
  idlocalitat NUMBER CONSTRAINT NN_CL_idlocalitat NOT NULL
  CONSTRAINT FK_loc_CL_idlocalitat REFERENCES localitat(idlocalitat),
  data_alta DATE CONSTRAINT NN_CL_data_alta NOT NULL,
  data_baixa DATE,
  data_last_mod DATE
);
/

-- COMPTADOR --
CREATE TABLE comptador(
  idContracte NUMBER CONSTRAINT PK_comptador PRIMARY KEY, 
  NIF VARCHAR2 (10 BYTE) CONSTRAINT NN_CMP_NIF NOT NULL
  CONSTRAINT FK_CMP_NIF REFERENCES client(NIF),
  potencia NUMBER CONSTRAINT NN_CMP_potencia NOT NULL,
  direccio VARCHAR2(100 BYTE) CONSTRAINT NN_CMP_direccio NOT NULL,
  idlocalitat NUMBER CONSTRAINT NN_CMP_idlocalitat NOT NULL
  CONSTRAINT FK_CMP_idlocalitat REFERENCES localitat(idlocalitat),
  consum_actual NUMBER,
  consum_total NUMBER,
  idModel NUMBER CONSTRAINT NN_CMP_idModel NOT NULL
  CONSTRAINT FK_CMP_idModel REFERENCES model_comptador(idModel),
  darrera_lectura DATE,
  data_alta DATE CONSTRAINT NN_CMP_data_alta NOT NULL,
  data_baixa DATE,
  data_last_mod DATE
);
/

CREATE SEQUENCE seq_COMPTADOR INCREMENT BY 1 START WITH 1;
/

CREATE OR REPLACE TRIGGER TG_Ins_idContracte
  BEFORE INSERT ON COMPTADOR
  FOR EACH ROW
BEGIN
  SELECT seq_COMPTADOR.NEXTVAL INTO :NEW.idContracte
  FROM DUAL;
END TG_Ins_idContracte;
/

-- DISTRIBUCIO_COMPTADOR --
CREATE TABLE distribucio_comptador(
  idContracte NUMBER CONSTRAINT FK_DC_idcontracte REFERENCES comptador(idContracte), 
  idCentraldis NUMBER CONSTRAINT FK_DC_idcentraldis REFERENCES central_distribucio(idCentraldis), 
  CONSTRAINT PK_distribucio_comptador PRIMARY KEY (idContracte,idCentraldis)
);
/

-- LECTURA_COMPTADOR --
CREATE TABLE lectura_comptador(
  idContracte NUMBER CONSTRAINT NN_LCM_idContracte NOT NULL
  CONSTRAINT FK_LCM_idContracte REFERENCES comptador(idContracte),
  data_lectura DATE,
  data_inici DATE,
  data_fi DATE,
  valor_lectura NUMBER CONSTRAINT NN_LCM_valor_lectura NOT NULL,
  idmodalitat NUMBER CONSTRAINT NN_LCM_idmodalitat NOT NULL
  CONSTRAINT FK_LCM_idmodalitat REFERENCES modalitat_lectura(idmodalitat),
  CONSTRAINT PK_lectura_comptador PRIMARY KEY (idContracte,data_lectura,data_inici,data_fi) 
);
/


-- Modul Estadstic -- 
CREATE TABLE est_consumcomptprod(
    idCentralprod NUMBER CONSTRAINT PK_est_consumcomptprod PRIMARY KEY,
    consum_total NUMBER CONSTRAINT NN_ECCP_consum_total NOT NULL
);
/

CREATE TABLE est_consumliniaany(
    idlinia NUMBER,
    p_any NUMBER,
    consum NUMBER,
    num_compt NUMBER,
    mitja_consum NUMBER,
    CONSTRAINT PK_est_consumliniaany PRIMARY KEY (idlinia,p_any) 
);
/

CREATE TABLE est_liniamescarregada(
    idlinia NUMBER CONSTRAINT PK_est_liniamescarregada PRIMARY KEY,
    carrega_total NUMBER,
    capacitat_max NUMBER,
    perc_carrega NUMBER
);
/

CREATE TABLE est_anyliniesenconsum(
    p_any NUMBER CONSTRAINT PK_est_anyliniesenconsum PRIMARY KEY,
    num_linies NUMBER,
    superen NUMBER,
    perc_linies NUMBER
);
/

CREATE TABLE est_consumcpany(
    idcentralprod NUMBER,
    p_any NUMBER,
    energia_any NUMBER,
    consum NUMBER,
    perc_consum NUMBER,
    CONSTRAINT PK_est_consumcpany PRIMARY KEY (idcentralprod,p_any) 
);

CREATE TABLE est_sumacpany(
    p_any NUMBER CONSTRAINT PK_est_sumacpany PRIMARY KEY,
    total NUMBER CONSTRAINT NN_EA_total NOT NULL
);
/

CREATE TABLE est_top10comptadors(
    idContracte NUMBER CONSTRAINT PK_est_top10comptadors PRIMARY KEY,
    consum_total NUMBER CONSTRAINT NN_T10C_consum_total NOT NULL
);
/

CREATE TABLE est_consummigclients(
    consum_mig NUMBER CONSTRAINT PK_est_consum_mig_clients PRIMARY KEY,
    consum_total NUMBER,
    num_clients NUMBER
);
/

-- Mdul Control --
CREATE TABLE logs(
  idlog NUMBER CONSTRAINT PK_logs PRIMARY KEY, 
  nomproc VARCHAR2(100) CONSTRAINT NN_nomproc NOT NULL,
  logdate DATE CONSTRAINT NN_logdate NOT NULL,
  parin VARCHAR2(500 BYTE),
  parout VARCHAR2(500 BYTE)
);
/

CREATE SEQUENCE seq_LOGS INCREMENT BY 1 START WITH 1;

CREATE OR REPLACE TRIGGER TG_Ins_idlog
  BEFORE INSERT ON LOGS
  FOR EACH ROW
BEGIN
  SELECT seq_LOGS.NEXTVAL INTO :NEW.idlog
  FROM DUAL;
END TG_Ins_idlog;
/

COMMIT;
/

spool off;
/