SPOOL pkg_ABM.log

CREATE OR REPLACE PACKAGE pkg_abm
AS
/*******************************************************************************************
   NOM:      pkg_ABM
   PROPSIT: Manteniment d'Altes, Baixes i Modificacions (ABM) de les taules:
                      - Central_produccio
                      - Central_distribucio
                      - Comptador
                      - Linia_comunicacio
                      - Client
                      - Poblacio
                      - Localitat
********************************************************************************************/

/******************************** Central_produccio ****************************************/

   PROCEDURE prc_alta_central_produccio (
      p_idtipusprod   IN       central_produccio.idtipusprod%TYPE,
      p_direccio      IN       central_produccio.direccio%TYPE,
      p_idlocalitat   IN       central_produccio.idlocalitat%TYPE,
      p_data_it       IN       central_produccio.data_it%TYPE,
      p_atribut_1     IN       central_produccio.atribut_1%TYPE,
      p_atribut_2     IN       central_produccio.atribut_2%TYPE,
      RSP             OUT      VARCHAR2
   );
/*******************************************************************************************
    NOM:        prc_alta_central_produccio
    PROPSIT:   Dona d'alta una central de producci
    REQUISITS:  - Han d'estar informats tots els camps obligatoris (NOT NULL)
                - El registre no ha d'existir
                - Els Id's de linia, localitat i tipus han d'existir a la base de dades
    PARMETRES:
        IN:   Obligatoris:  p_idtipusprod: Identificador del tipus de central de producci
                            p_idlinia: Identificador de la linia de comunicaci
                            p_direccio: Adrea de la central de producci
                            p_idlocalitat: Identificador de la localitat de la direcci
              Opcionals:    p_data_IT: Data de la darrera Inspecci Tcnica (IT)
                            p_atribut_1: Aquest valor vindr definit depenent del tipus
                            (nuclear, trmica...)
                            p_atribut_2: En cas de ser una central de tipus nuclear, definim 
                            l'atribut 2 com els Kg de rebuig radioactiu generat.
        OUT:  RSP
    PRE-CONDICI:  No existeix la central de producci
    POST-CONDICI: Se dona d'alta la central de producci
    RETORNA:
        tot ha anat be:
            RSP = 'OK: alta de central de producci realitzada correctament. Id = NumId'
        missatge de errors:
            'ERROR: hi ha camps obligatoris sense informar'
            'ERROR: La central de producci ja existeix'
            'ERROR: El tipus de central no existeix a la base de dades'
            'ERROR: La linia de comunicaci no existeix a la base de dades'
            'ERROR: La localitat no existeix a la base de dades'
            'ERROR: Error no tractat'
*******************************************************************************************/

   PROCEDURE prc_baixa_central_produccio (
      p_idcentralprod   IN       central_produccio.idcentralprod%TYPE,
      RSP               OUT      VARCHAR2
   );
/*******************************************************************************************
    NOM:        prc_baixa_central_produccio
    PROPSIT:   Dona de baixa una central de producci
    REQUISITS:  - S'ha d'informar l'Id de la central de producci
                - El registre ha d'existir a la base de dades
    PARMETRES:
        IN:   Obligatoris:  p_idcentralprod: Identificador de la central de producci
        OUT:  RSP
    PRE-CONDICI:  Existeix la central de producci que se vol donar de baixa
    POST-CONDICI: se dona de baixa la central de producci
    RETORNA:
        tot ha anat be:
            RSP = 'OK: baixa de central de producci realitzada correctament'
        missatge de errors:
            'ERROR: hi ha camps obligatoris sense informar'
            'ERROR: La central de producci no existeix'
            'ERROR: El registre ja est donat de baixa'
            'ERROR: Error no tractat'
*******************************************************************************************/

   PROCEDURE prc_modif_central_produccio (
      p_idcentralprod   IN       central_produccio.idcentralprod%TYPE,
      p_idtipusprod     IN       central_produccio.idtipusprod%TYPE,
      p_direccio        IN       central_produccio.direccio%TYPE,
      p_idlocalitat     IN       central_produccio.idlocalitat%TYPE,
      p_data_it         IN       central_produccio.data_it%TYPE,
      p_atribut_1       IN       central_produccio.atribut_1%TYPE,
      p_atribut_2       IN       central_produccio.atribut_2%TYPE,
      RSP               OUT      VARCHAR2
   );
/*******************************************************************************************
    NOM:        prc_modif_central_produccio
    PROPSIT:   Modifica una central de producci
    REQUISITS:  - S'ha d'informar l'Id de la central de producci
                - El registre ha d'existir a la base de dades
                - Existeix el tipus de central de producci que se vol modificar
                - Existeix la linia de comunicaci que se vol modificar
                - Existeix la localitat que se vol modificar
    PARMETRES:
        IN:   Obligatoris:  p_idcentralprod: Identificador de la central de producci
              Opcionals:    p_idtipusprod: Identificador del tipus de central de producci
                            p_idlinia: Identificador de la lnia de comunicaci
                            p_direccio: Adrea de la central de producci
                            p_idlocalitat: Identificador de la localitat de la direcci
                            p_data_IT: Data de la darrera Inspecci Tcnica (IT)
                            p_atribut_1: Aquest valor vindr definit depenent del tipus
                            (nuclear, trmica...)
                            p_atribut_2: En cas de ser una central de tipus nuclear, definim 
                            l'atribut 2 com els Kg de rebuig radioactiu generat.
        OUT:  RSP
    PRE-CONDICI:  Existeix la central de producci que se vol modificar
    POST-CONDICI: se modifica la central de producci
    RETORNA:
        tot ha anat be:
            RSP = 'OK: modificaci de central de producci realitzada correctament'
        missatge de errors:
            'ERROR: hi ha camps obligatoris sense informar'
            'ERROR: La central de producci no existeix'
            'ERROR: El tipus de central no existeix a la base de dades'
            'ERROR: La linia de comunicaci no existeix a la base de dades'
            'ERROR: La localitat no existeix a la base de dades'
            'ERROR: Error no tractat'
*******************************************************************************************/

   PROCEDURE prc_react_central_produccio (
      p_idcentralprod   IN       central_produccio.idcentralprod%TYPE,
      RSP               OUT      VARCHAR2
   );
/*******************************************************************************************
    NOM:        prc_react_central_produccio
    PROPSIT:   Reactiva una central de producci donada de baixa
    REQUISITS:  - S'ha d'informar l'Id de la central de producci
                - El registre ha d'existir a la base de dades
    PARMETRES:
        IN:   Obligatoris:  p_idcentralprod: Identificador de la central de producci
        OUT:  RSP
    PRE-CONDICI:  Existeix la central de producci que se vol reactivar
    POST-CONDICI: se reactiva la central de producci
    RETORNA:
        tot ha anat be:
            RSP = 'OK: reactivaci de central de producci realitzada correctament'
        missatge de errors:
            'ERROR: hi ha camps obligatoris sense informar'
            'ERROR: La central de producci no existeix'
            'ERROR: El registre ja est donat d'alta'
            'ERROR: Error no tractat'
*******************************************************************************************/

/******************************** Central_distribucio **************************************/

   PROCEDURE prc_alta_central_distribucio (
      p_idlinia         IN       central_distribucio.idlinia%TYPE,
      p_direccio        IN       central_distribucio.direccio%TYPE,
      p_idlocalitat     IN       central_distribucio.idlocalitat%TYPE,
      p_energia_max     IN       central_distribucio.energia_max%TYPE,
      RSP               OUT      VARCHAR2
   );
/*******************************************************************************************
    NOM:        prc_alta_central_distribucio
    PROPSIT:   Dona d'alta una central de distribuci
    REQUISITS:  - Han d'estar informats tots els camps obligatoris (NOT NULL)
                - El registre no ha d'existir
                - Els Id's de central de producci i localitat han d'existir a la base de dades
    PARMETRES:
        IN:   Obligatoris:  p_idcentralprod: Identificador de la central de producci
                            p_direccio: Adrea de la central de producci
                            p_idlocalitat: Identificador de la localitat de la direcci
                            p_energia_max: Energia mxima produda (KWh)
        OUT:  RSP
    PRE-CONDICI:  No existeix la central de distribuci
    POST-CONDICI: Se dona d'alta la central de distribuci
    RETORNA:
        tot ha anat be:
            RSP = 'OK: alta de central de distribucio realitzada correctament. Id = NumId'
        missatge de errors:
            'ERROR: hi ha camps obligatoris sense informar'
            'ERROR: La central de distribuci ja existeix'
            'ERROR: La linia de comunicaci no existeix a la base de dades'
            'ERROR: La localitat no existeix a la base de dades'
            'ERROR: Error no tractat'
*******************************************************************************************/

   PROCEDURE prc_baixa_central_distribucio (
      p_idcentraldis    IN       central_distribucio.idcentraldis%TYPE,
      RSP               OUT      VARCHAR2
   );
/*******************************************************************************************
    NOM:        prc_baixa_central_distribucio
    PROPSIT:   Dona de baixa una central de distribucio
    REQUISITS:  - S'ha d'informar l'Id de la central de distribucio
                - El registre ha d'existir a la base de dades
    PARMETRES:
        IN:   Obligatoris:  p_idcentraldis: Identificador de la central de distribucio
        OUT:  RSP
    PRE-CONDICI:  Existeix el la central de distribucio que se vol donar de baixa
    POST-CONDICI: se dona de baixa la central de distribucio
    RETORNA:
        tot ha anat be:
            RSP = 'OK: baixa de central de distribucio realitzada correctament'
        missatge de errors:
            'ERROR: hi ha camps obligatoris sense informar'
            'ERROR: La central de distribucio no existeix'
            'ERROR: Error no tractat'
*******************************************************************************************/

   PROCEDURE prc_modif_central_distribucio (
      p_idcentraldis    IN       central_distribucio.idcentraldis%TYPE,
      p_idlinia         IN       central_distribucio.idlinia%TYPE,
      p_direccio        IN       central_distribucio.direccio%TYPE,
      p_idlocalitat     IN       central_distribucio.idlocalitat%TYPE,
      p_energia_max     IN       central_distribucio.energia_max%TYPE,
      RSP               OUT      VARCHAR2
   );
/*******************************************************************************************
    NOM:        prc_modif_central_distribuci
    PROPSIT:   Modifica una central de distribuci
    REQUISITS:  - S'ha d'informar l'Id de la central de distribuci
                - El registre ha d'existir a la base de dades
                - Existeix la de central de producci que se vol modificar
                - Existeix la localitat que se vol modificar
    PARMETRES:
        IN:   Obligatoris:  p_idcentraldis: Identificador de la central de distribuci
              Opcionals:    p_idcentralprod: Identificador de la central de producci
                            p_direccio: Adrea de la central de producci
                            p_idlocalitat: Identificador de la localitat de la direcci
                            p_energia_max: Energia mxima produda (KWh)
        OUT:  RSP
    PRE-CONDICI:  Existeix la central de distribuci que se vol modificar
    POST-CONDICI: se modifica la central de distribuci
    RETORNA:
        tot ha anat be:
            RSP = 'OK: modificaci de central de distribuci realitzada correctament'
        missatge de errors:
            'ERROR: hi ha camps obligatoris sense informar'
            'ERROR: La central de distribuci no existeix'
            'ERROR: La central de producci no existeix a la base de dades'
            'ERROR: La localitat no existeix a la base de dades'
            'ERROR: Error no tractat'
*******************************************************************************************/

   PROCEDURE prc_react_central_distribucio (
      p_idcentraldis    IN       central_distribucio.idcentraldis%TYPE,
      RSP               OUT      VARCHAR2
   );
/*******************************************************************************************
    NOM:        prc_react_central_distribucio
    PROPSIT:   Reactiva una central de distribucio
    REQUISITS:  - S'ha d'informar l'Id de la central de distribucio
                - El registre ha d'existir a la base de dades
    PARMETRES:
        IN:   Obligatoris:  p_idcentraldis: Identificador de la central de distribucio
        OUT:  RSP
    PRE-CONDICI:  Existeix el la central de distribucio que se vol reactivar
    POST-CONDICI: se reactiva la central de distribucio
    RETORNA:
        tot ha anat be:
            RSP = 'OK: reactivaci de central de distribucio realitzada correctament'
        missatge de errors:
            'ERROR: hi ha camps obligatoris sense informar'
            'ERROR: La central de distribucio no existeix'
            'ERROR: El registre ja est donat d'alta'
            'ERROR: Error no tractat'
*******************************************************************************************/

/************************************** Comptador *****************************************/

   PROCEDURE prc_alta_comptador (
      p_nif               IN       comptador.nif%TYPE,
      p_potencia          IN       comptador.potencia%TYPE,
      p_direccio          IN       comptador.direccio%TYPE,
      p_idlocalitat       IN       comptador.idlocalitat%TYPE,
      p_idmodel           IN       comptador.idmodel%TYPE,
      RSP                 OUT      VARCHAR2
   );
/*******************************************************************************************
    NOM:        prc_alta_comptador
    PROPSIT:   Dona d'alta un comptador
    REQUISITS:  - Han d'estar informats tots els camps obligatoris (NOT NULL)
                - El registre no ha d'existir
                - Els Id's de central distribuci, NIF (client), localitat i model han d'existir a la base de dades
    PARMETRES:
        IN:   Obligatoris:  p_nif: NIF del client
                            p_potencia: potncia contractada pels clients
                            p_direccio: Adrea del comptador
                            p_idlocalitat: Identificador de la localitat de la direcci
                            p_idmodel: Identificador del model del comptador
        OUT:  RSP
    PRE-CONDICI:  No existeix el comptador
    POST-CONDICI: Se dona d'alta el comptador
    RETORNA:
        tot ha anat be:
            RSP = 'OK: alta de comptador realitzada correctament. Id = NumId'
        missatge de errors:
            'ERROR: hi ha camps obligatoris sense informar'
            'ERROR: El comptador ja existeix'
            'ERROR: El client no existeix a la base de dades'
            'ERROR: La localitat no existeix a la base de dades'
            'ERROR: Error no tractat'
*******************************************************************************************/

   PROCEDURE prc_baixa_comptador (
      p_idcontracte     IN  comptador.idcontracte%TYPE,
      RSP               OUT VARCHAR2
   );
/*******************************************************************************************
    NOM:        prc_baixa_comptador
    PROPSIT:   Dona de baixa un comptador
    REQUISITS:  - S'ha d'informar l'Id del comptador
                - El registre ha d'existir a la base de dades
    PARMETRES:
        IN:   Obligatoris:  p_idcontracte: Identificador del comptador
        OUT:  RSP
    PRE-CONDICI:  Existeix el comptador que se vol donar de baixa
    POST-CONDICI: se dona de baixa el comptador
    RETORNA:
        tot ha anat be:
            RSP = 'OK: baixa de comptador realitzada correctament'
        missatge de errors:
            'ERROR: hi ha camps obligatoris sense informar'
            'ERROR: El comptador no existeix'
            'ERROR: Error no tractat'
*******************************************************************************************/

   PROCEDURE prc_modif_comptador (
      p_idcontracte       IN       comptador.idcontracte%TYPE,
      p_nif               IN       comptador.nif%TYPE,
      p_potencia          IN       comptador.potencia%TYPE,
      p_direccio          IN       comptador.direccio%TYPE,
      p_idlocalitat       IN       comptador.idlocalitat%TYPE,
      p_idmodel           IN       comptador.idmodel%TYPE,
      RSP                 OUT      VARCHAR2
   );
/*******************************************************************************************
    NOM:        prc_modif_comptador
    PROPSIT:   Modifica un comptador
    REQUISITS:  - S'ha d'informar l'Id del comptador
                - El registre ha d'existir a la base de dades
                - Els Id's de central distribuci, NIF (client), localitat i model han d'existir a la base de dades
    PARMETRES:
        IN:   Obligatoris:  p_idcontracte: Identificador del comptador
                            p_nif: NIF del client
                            p_potencia: potncia contractada pels clients
                            p_direccio: Adrea del comptador
                            p_idlocalitat: Identificador de la localitat de la direcci
                            p_idmodel: Identificador del model del comptador
        OUT:  RSP
    PRE-CONDICI:  Existeix el comptador que se vol modificar 
    POST-CONDICI: Se modifica el comptador
    RETORNA:
        tot ha anat be:
            RSP = 'OK: alta de comptador realitzada correctament. Id = NumId'
        missatge de errors:
            'ERROR: hi ha camps obligatoris sense informar'
            'ERROR: El comptador no existeix'
            'ERROR: La central de distribuci no existeix a la base de dades'
            'ERROR: El client no existeix a la base de dades'
            'ERROR: La localitat no existeix a la base de dades'
            'ERROR: Error no tractat'
*******************************************************************************************/

   PROCEDURE prc_react_comptador (
      p_idcontracte     IN  comptador.idcontracte%TYPE,
      RSP               OUT VARCHAR2
   );
/*******************************************************************************************
    NOM:        prc_react_comptador
    PROPSIT:   Reactiva un comptador
    REQUISITS:  - S'ha d'informar l'Id del comptador
                - El registre ha d'existir a la base de dades
    PARMETRES:
        IN:   Obligatoris:  p_idcontracte: Identificador del comptador
        OUT:  RSP
    PRE-CONDICI:  Existeix el comptador que se vol donar reactivar
    POST-CONDICI: se reactiva el comptador
    RETORNA:
        tot ha anat be:
            RSP = 'OK: reactivaci de comptador realitzada correctament'
        missatge de errors:
            'ERROR: hi ha camps obligatoris sense informar'
            'ERROR: El comptador no existeix'
            'ERROR: El registre ja est donat d'alta'
            'ERROR: Error no tractat'
*******************************************************************************************/
  
/********************************** Linia_comunicacio **************************************/

   PROCEDURE prc_alta_linia_comunicacio (
      p_idcentralprod     IN       linia_comunicacio.idcentralprod%TYPE,
      p_capacitat_max     IN       linia_comunicacio.capacitat_max%TYPE,
      RSP                 OUT      VARCHAR2
   );
/*******************************************************************************************
    NOM:        prc_alta_linia_comunicacio
    PROPSIT:   Dona d'alta una lnia de comunicaci
    REQUISITS:  - Han d'estar informats tots els camps obligatoris (NOT NULL)
                - El registre no ha d'existir
    PARMETRES:
        IN:   Obligatoris:  p_capacitat_max: Capacitat mxima de la lnia                            
        OUT:  RSP
    PRE-CONDICI:  No existeix la lnia de comunicaci
    POST-CONDICI: Se dona d'alta la lnia de comunicaci
    RETORNA:
        tot ha anat be:
            RSP = 'OK: alta de lnia realitzada correctament. Id = NumId'
        missatge de errors:
            'ERROR: hi ha camps obligatoris sense informar'
            'ERROR: Error no tractat'
*******************************************************************************************/

   PROCEDURE prc_baixa_linia_comunicacio (
      p_idlinia         IN  linia_comunicacio.idlinia%TYPE,
      RSP               OUT VARCHAR2
   );
/*******************************************************************************************
    NOM:        prc_baixa_linia_comunicacio
    PROPSIT:   Dona de baixa una lnia de comunicaci
    REQUISITS:  - S'ha d'informar l'Id de la lnia de comunicaci
                - El registre ha d'existir a la base de dades
    PARMETRES:
        IN:   Obligatoris:  p_idlinia: Identificador de la lnia de comunicaci
        OUT:  RSP
    PRE-CONDICI:  Existeix la lnia de comunicaci que se vol donar de baixa
    POST-CONDICI: se dona de baixa la lnia de comunicaci
    RETORNA:
        tot ha anat be:
            RSP = 'OK: baixa de lnia de comunicaci realitzada correctament'
        missatge de errors:
            'ERROR: hi ha camps obligatoris sense informar'
            'ERROR: La lnia de comunicaci no existeix'
            'ERROR: Error no tractat'
*******************************************************************************************/

   PROCEDURE prc_modif_linia_comunicacio (
      p_idlinia           IN       linia_comunicacio.idlinia%TYPE,
      p_idcentralprod     IN       linia_comunicacio.idcentralprod%TYPE,
      p_capacitat_max     IN       linia_comunicacio.capacitat_max%TYPE,
      RSP                 OUT      VARCHAR2
   );
/*******************************************************************************************
    NOM:        prc_modif_linia_comunicacio
    PROPSIT:   Modifica una lnia de comunicaci
    REQUISITS:  - S'ha d'informar l'Id de la lnia de comunicaci
    PARMETRES:
        IN:   Obligatoris:  p_idlinia: Identificador de la lnia de comunicaci
              Opcionals:    p_capacitat_max: Capacitat mxima de la lnia 
        OUT:  RSP
    PRE-CONDICI:  Existeix la lnia de comunicaci que se vol modificar 
    POST-CONDICI: Se modifica la lnia de comunicaci
    RETORNA:
        tot ha anat be:
            RSP = 'OK: modificaci de lnia de comunicaci realitzada correctament'
        missatge de errors:
            'ERROR: hi ha camps obligatoris sense informar'
            'ERROR: la lnia de comunicaci no existeix'
            'ERROR: Error no tractat'
*******************************************************************************************/

   PROCEDURE prc_react_linia_comunicacio (
      p_idlinia         IN  linia_comunicacio.idlinia%TYPE,
      RSP               OUT VARCHAR2
   );
/*******************************************************************************************
    NOM:        prc_react_linia_comunicacio
    PROPSIT:   Reactiva una lnia de comunicaci
    REQUISITS:  - S'ha d'informar l'Id de la lnia de comunicaci
                - El registre ha d'existir a la base de dades
    PARMETRES:
        IN:   Obligatoris:  p_idlinia: Identificador de la lnia de comunicaci
        OUT:  RSP
    PRE-CONDICI:  Existeix la lnia de comunicaci que se vol reactivar
    POST-CONDICI: se reactiva la lnia de comunicaci
    RETORNA:
        tot ha anat be:
            RSP = 'OK: reactivaci de lnia de comunicaci realitzada correctament'
        missatge de errors:
            'ERROR: hi ha camps obligatoris sense informar'
            'ERROR: La lnia de comunicaci no existeix'
            'ERROR: El registre ja est donat d'alta'
            'ERROR: Error no tractat'
*******************************************************************************************/

/*************************************** Client ********************************************/

   PROCEDURE prc_alta_client (
      p_nif               IN       client.nif%TYPE,
      p_tipusclient       IN       client.tipus_client%TYPE,
      p_nom               IN       client.nom%TYPE,
      p_cognom1           IN       client.cognom1%TYPE,
      p_cognom2           IN       client.cognom2%TYPE,
      p_data_naixement    IN       client.data_naixement%TYPE,
      p_direccio          IN       client.direccio%TYPE,
      p_idlocalitat       IN       client.idlocalitat%TYPE,
      RSP                 OUT      VARCHAR2
   );
/*******************************************************************************************
    NOM:        prc_alta_client
    PROPSIT:   Dona d'alta un client
    REQUISITS:  - Han d'estar informats tots els camps obligatoris (NOT NULL)
                - El registre no ha d'existir
                - L'Id de localitat ha d'existir a la base de dades
    PARMETRES:
        IN:   Obligatoris:  p_nif: NIF(Identificador) del client
                            p_tipusclient: indica si el client es una persona (1) o una empresa (2)
                            p_nom: Nom del client
                            p_direccio: Adrea del client
                            p_idlocalitat: Identificador de la localitat de la direcci
              Opcionals:    p_data_naixement: Data de naixement de el client    
                            p_cognom1: Primer cognom del client
                            p_cognom2: Segon cognom del client             
        OUT:  RSP
    PRE-CONDICI:  No existeix el client
    POST-CONDICI: Se dona d'alta el client
    RETORNA:
        tot ha anat be:
            RSP = 'OK: alta de client realitzada correctament. Id = NIF'
        missatge de errors:
            'ERROR: hi ha camps obligatoris sense informar'
            'ERROR: El client ja existeix'
            'ERROR: La localitat no existeix a la base de dades'
            'ERROR: Error no tractat'
*******************************************************************************************/ 

   PROCEDURE prc_baixa_client (
      p_nif             IN  client.nif%TYPE,
      RSP               OUT VARCHAR2
   );
/*******************************************************************************************
    NOM:        prc_baixa_client
    PROPSIT:   Dona de baixa un client
    REQUISITS:  - S'ha d'informar el NIF del client
                - El registre ha d'existir a la base de dades
    PARMETRES:
        IN:   Obligatoris:  p_nif: NIF(Identificador) del client
        OUT:  RSP
    PRE-CONDICI:  Existeix el client que se vol donar de baixa
    POST-CONDICI: se dona de baixa el client
    RETORNA:
        tot ha anat be:
            RSP = 'OK: baixa de client realitzada correctament'
        missatge de errors:
            'ERROR: hi ha camps obligatoris sense informar'
            'ERROR: El client no existeix'
            'ERROR: Error no tractat'
*******************************************************************************************/

   PROCEDURE prc_modif_client (
      p_nif               IN       client.nif%TYPE,
      p_tipusclient       IN       client.tipus_client%TYPE,
      p_nom               IN       client.nom%TYPE,
      p_cognom1           IN       client.cognom1%TYPE,
      p_cognom2           IN       client.cognom2%TYPE,
      p_data_naixement    IN       client.data_naixement%TYPE,
      p_direccio          IN       client.direccio%TYPE,
      p_idlocalitat       IN       client.idlocalitat%TYPE,
      RSP                 OUT      VARCHAR2
   );
/*******************************************************************************************
    NOM:        prc_modif_client
    PROPSIT:   Modifica un client
    REQUISITS:  - S'ha d'informar l'Id del client
                - L'Id de localitat ha d'existir a la base de dades
    PARMETRES:
        IN:   Obligatoris:  p_nif: NIF(Identificador) del client
              Opcionals:    p_tipusclient: indica si el client es una persona (1) o una empresa (2)
                            p_nom: Nom del client
                            p_cognom1: Primer cognom del client
                            p_cognom2: Segon cognom del client
                            p_direccio: Adrea del client
                            p_idlocalitat: Identificador de la localitat de la direcci
                            p_data_naixement: Data de naixement de el client
        OUT:  RSP
    PRE-CONDICI:  Existeix el client que se vol modificar 
    POST-CONDICI: Se modifica el client
    RETORNA:
        tot ha anat be:
            RSP = 'OK: modificaci de client realitzada correctament'
        missatge de errors:
            'ERROR: hi ha camps obligatoris sense informar'
            'ERROR: El client no existeix'
            'ERROR: La localitat no existeix a la base de dades'
            'ERROR: Error no tractat'
*******************************************************************************************/

   PROCEDURE prc_react_client (
      p_nif             IN  client.nif%TYPE,
      RSP               OUT VARCHAR2
   );
/*******************************************************************************************
    NOM:        prc_react_client
    PROPSIT:   Reactiva un client
    REQUISITS:  - S'ha d'informar el NIF del client
                - El registre ha d'existir a la base de dades
    PARMETRES:
        IN:   Obligatoris:  p_nif: NIF(Identificador) del client
        OUT:  RSP
    PRE-CONDICI:  Existeix el client que se vol reactivar
    POST-CONDICI: se reactiva el client
    RETORNA:
        tot ha anat be:
            RSP = 'OK: reactivaci de client realitzada correctament'
        missatge de errors:
            'ERROR: hi ha camps obligatoris sense informar'
            'ERROR: El client no existeix'
            'ERROR: El registre ja est donat d'alta'
            'ERROR: Error no tractat'
*******************************************************************************************/

/************************************** Poblacio *******************************************/

   PROCEDURE prc_alta_poblacio (
      p_nompoblacio       IN       poblacio.nompoblacio%TYPE,
      p_idpais            IN       poblacio.idpais%TYPE,
      RSP                 OUT      VARCHAR2
   );
/*******************************************************************************************
    NOM:        prc_alta_poblacio
    PROPSIT:   Dona d'alta una poblaci
    REQUISITS:  - Han d'estar informats tots els camps obligatoris (NOT NULL)
                - El registre no ha d'existir
                - L'Id de pas ha d'existir a la base de dades
    PARMETRES:
        IN:   Obligatoris:  p_nompoblacio: Nom de la poblaci
                            p_idpais: Identificador del pas         
        OUT:  RSP
    PRE-CONDICI:  No existeix la poblaci
    POST-CONDICI: Se dona d'alta la poblaci
    RETORNA:
        tot ha anat be:
            RSP = 'OK: alta de poblaci realitzada correctament. Id = NumId'
        missatge de errors:
            'ERROR: hi ha camps obligatoris sense informar'
            'ERROR: La poblaci ja existeix'
            'ERROR: El pas no existeix a la base de dades'
            'ERROR: Error no tractat'
*******************************************************************************************/

   PROCEDURE prc_borra_poblacio (
      p_idpoblacio        IN       poblacio.idpoblacio%TYPE,
      RSP                 OUT      VARCHAR2
   );
/*******************************************************************************************
    NOM:        prc_borra_poblacio
    PROPSIT:   Esborra una poblaci
    REQUISITS:  - S'ha d'informar l'Id de la poblaci
                - El registre ha d'existir a la base de dades
                - El registre no ha d'estar lligat a cap taula externa (localitat)
    PARMETRES:
        IN:   Obligatoris:  p_idpoblacio: Identificador de la poblaci
        OUT:  RSP
    PRE-CONDICI:  Existeix la poblaci que se vol borrar
    POST-CONDICI: se borra la poblaci
    RETORNA:
        tot ha anat be:
            RSP = 'OK: Delete de poblaci realitzat correctament'
        missatge de errors:
            'ERROR: hi ha camps obligatoris sense informar'
            'ERROR: La poblaci no existeix'
            'ERROR: Hi ha localitats lligades a aquesta poblaci'
            'ERROR: Error no tractat'
*******************************************************************************************/

   PROCEDURE prc_modif_poblacio (
      p_idpoblacio        IN       poblacio.idpoblacio%TYPE,
      p_nompoblacio       IN       poblacio.nompoblacio%TYPE,
      p_idpais            IN       poblacio.idpais%TYPE,
      RSP                 OUT      VARCHAR2
   );
/*******************************************************************************************
    NOM:        prc_modif_poblacio
    PROPSIT:   Modifica una poblaci
    REQUISITS:  - S'ha d'informar l'Id de la poblaci
                - L'Id del pas ha d'existir a la base de dades
    PARMETRES:
        IN:   Obligatoris:  p_idpoblacio: Identificador de la poblaci
              Opcionals:    p_nompoblacio: Nom de la poblaci
                            p_idpais: Identificador del pas 
        OUT:  RSP
    PRE-CONDICI:  Existeix la poblaci que se vol modificar 
    POST-CONDICI: Se modifica la poblaci
    RETORNA:
        tot ha anat be:
            RSP = 'OK: modificaci de poblaci realitzada correctament'
        missatge de errors:
            'ERROR: hi ha camps obligatoris sense informar'
            'ERROR: La poblaci no existeix'
            'ERROR: El pas no existeix a la base de dades'
            'ERROR: Error no tractat'
*******************************************************************************************/
   
/************************************* Localitat *******************************************/

   PROCEDURE prc_alta_localitat (
      p_nomlocalitat      IN       localitat.nomlocalitat%TYPE, 
      p_idpoblacio        IN       localitat.idpoblacio%TYPE,
      p_codpostal         IN       localitat.codpostal%TYPE,
      RSP                 OUT      VARCHAR2
   );
/*******************************************************************************************
    NOM:        prc_alta_localitat
    PROPSIT:   Dona d'alta una localitat
    REQUISITS:  - Han d'estar informats tots els camps obligatoris (NOT NULL)
                - El registre no ha d'existir
                - L'Id de poblaci ha d'existir a la base de dades
    PARMETRES:
        IN:   Obligatoris:  p_idpoblacio: Identificador de la poblaci
                            p_nomlocalitat: Nom de la localitat
                            p_codpostal: Codi postal de la localitat       
        OUT:  RSP
    PRE-CONDICI:  No existeix la localitat
    POST-CONDICI: Se dona d'alta la localitat
    RETORNA:
        tot ha anat be:
            RSP = 'OK: alta de localitat realitzada correctament. Id = NumId'
        missatge de errors:
            'ERROR: hi ha camps obligatoris sense informar'
            'ERROR: La localitat ja existeix'
            'ERROR: La poblaci no existeix a la base de dades'
            'ERROR: Error no tractat'
*******************************************************************************************/

   PROCEDURE prc_borra_localitat (
      p_idlocalitat     IN  localitat.idlocalitat%TYPE,
      RSP               OUT VARCHAR2
   );
/*******************************************************************************************
    NOM:        prc_borra_localitat
    PROPSIT:   Borra una localitat
    REQUISITS:  - S'ha d'informar l'Id de la localitat
                - El registre ha d'existir a la base de dades
                - El registre no ha d'estar lligat a cap taula externa 
                  (central_produccio, central_distribucio, comptador, client)
    PARMETRES:
        IN:   Obligatoris:  p_idlocalitat: Identificador de la localitat
        OUT:  RSP
    PRE-CONDICI:  Existeix la localitat que se vol borrar
    POST-CONDICI: se borra la localitat
    RETORNA:
        tot ha anat be:
            RSP = 'OK: delete de localitat realitzat correctament'
        missatge de errors:
            'ERROR: hi ha camps obligatoris sense informar'
            'ERROR: La localitat no existeix'
            'ERROR: Hi ha centrals de producci lligades a aquesta localitat'
            'ERROR: Hi ha centrals de distribuci lligades a aquesta localitat'
            'ERROR: Hi ha comptadors lligats a aquesta localitat'
            'ERROR: Hi ha clients lligats a aquesta localitat'
            'ERROR: Error no tractat'
*******************************************************************************************/

   PROCEDURE prc_modif_localitat (
      p_idlocalitat       IN       localitat.idlocalitat%TYPE,
      p_idpoblacio        IN       localitat.idpoblacio%TYPE,
      p_nomlocalitat      IN       localitat.nomlocalitat%TYPE,
      p_codpostal         IN       localitat.codpostal%TYPE,
      RSP                 OUT      VARCHAR2
   ); 
/*******************************************************************************************
    NOM:        prc_modif_localitat
    PROPSIT:   Modifica una localitat
    REQUISITS:  - S'ha d'informar l'Id de la localitat
                - L'Id de la poblaci ha d'existir a la base de dades
    PARMETRES:
        IN:   Obligatoris:  p_idlocalitat: Identificador de la localitat
              Opcionals:    p_idpoblacio: Identificador de la poblaci
                            p_nomlocalitat: Nom de la localitat
                            p_codpostal: Codi postal de la localitat 
        OUT:  RSP
    PRE-CONDICI:  Existeix la localitat que se vol modificar 
    POST-CONDICI: Se modifica la localitat
    RETORNA:
        tot ha anat be:
            RSP = 'OK: modificaci de localitat realitzada correctament'
        missatge de errors:
            'ERROR: hi ha camps obligatoris sense informar'
            'ERROR: La poblaci no existeix'
            'ERROR: La poblaci no existeix a la base de dades'
            'ERROR: Error no tractat'
*******************************************************************************************/
    
END pkg_abm;
/

CREATE OR REPLACE PACKAGE BODY pkg_abm
AS
   /******************************** Central_produccio ****************************************/

   PROCEDURE prc_alta_central_produccio (
      p_idtipusprod   IN     central_produccio.idtipusprod%TYPE,
      p_direccio      IN     central_produccio.direccio%TYPE,
      p_idlocalitat   IN     central_produccio.idlocalitat%TYPE,
      p_data_it       IN     central_produccio.data_it%TYPE,
      p_atribut_1     IN     central_produccio.atribut_1%TYPE,
      p_atribut_2     IN     central_produccio.atribut_2%TYPE,
      RSP                OUT VARCHAR2)
   IS
      /*******************************************************************************************
          NOM:        prc_alta_central_produccio
          PROPSIT:   Dona d'alta una central de producci
          REQUISITS:  - Han d'estar informats tots els camps obligatoris (NOT NULL)
                      - El registre no ha d'existir
                      - Els Id's de linia, localitat i tipus han d'existir a la base de dades
          PARMETRES:
              IN:   Obligatoris:  p_idtipusprod: Identificador del tipus de central de producci
                                  p_direccio: Adrea de la central de producci
                                  p_idlocalitat: Identificador de la localitat de la direcci
                    Opcionals:    p_data_IT: Data de la darrera Inspecci Tcnica (IT)
                                  p_atribut_1: Aquest valor vindr definit depenent del tipus
                                  (nuclear, trmica...)
                                  p_atribut_2: En cas de ser una central de tipus nuclear, definim
                                  l'atribut 2 com els Kg de rebuig radioactiu generat.
              OUT:  RSP
          PRE-CONDICI:  No existeix la central de producci
          POST-CONDICI: Se dona d'alta la central de producci
          RETORNA:
              tot ha anat be:
                  RSP = 'OK: alta de central de producci realitzada correctament. Id = NumId'
              missatge de errors:
                  'ERROR: hi ha camps obligatoris sense informar'
                  'ERROR: La central de producci ja existeix'
                  'ERROR: El tipus de central no existeix a la base de dades'
                  'ERROR: La localitat no existeix a la base de dades'
                  'ERROR: Error no tractat'
      *******************************************************************************************/
      err_null       EXCEPTION;
      err_no_tipus   EXCEPTION;
      err_no_loc     EXCEPTION;

      v_count        VARCHAR2 (1);
      v_parin        VARCHAR2 (300);
      v_seq          NUMBER;
      err_num        NUMBER;
      err_msg        VARCHAR2 (100);
   BEGIN
      v_parin :=
            'p_idtipusprod: '
         || p_idtipusprod
         || ', p_direccio: '
         || p_direccio
         || ', p_idlocalitat: '
         || p_idlocalitat
         || ', p_data_it: '
         || p_data_it
         || ', p_atribut_1'
         || p_atribut_1
         || ', p_atribut_2'
         || p_atribut_2;

      -- Verificaci de requisits
      -- Han d'estar informats tots els camps obligatoris (NOT NULL)
      IF (   TRIM (p_idtipusprod) IS NULL
          OR TRIM (p_direccio) IS NULL
          OR TRIM (p_idlocalitat) IS NULL)
      THEN
         RAISE err_null;
      END IF;

      -- Existeix Id a tipus_centralprod
      BEGIN
         v_count := NULL;

         SELECT 1
           INTO v_count
           FROM tipus_centralprod
          WHERE idtipusprod = p_idtipusprod AND ROWNUM = 1;
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            RAISE err_no_tipus;
      END;

      -- Existeix Id a localitat
      BEGIN
         v_count := NULL;

         SELECT 1
           INTO v_count
           FROM localitat
          WHERE idlocalitat = p_idlocalitat AND ROWNUM = 1;
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            RAISE err_no_loc;
      END;

      -- Si compleix els requisits, se fa la inserci
      INSERT INTO CENTRAL_PRODUCCIO (IDTIPUSPROD,
                                     DIRECCIO,
                                     IDLOCALITAT,
                                     ATRIBUT_1,
                                     ATRIBUT_2,
                                     DATA_IT,
                                     DATA_ALTA)
           VALUES (p_idtipusprod,
                   p_direccio,
                   p_idlocalitat,
                   p_atribut_1,
                   p_atribut_2,
                   p_data_IT,
                   SYSDATE);

      SELECT seq_CENTRAL_PRODUCCIO.CURRVAL INTO v_seq FROM DUAL;

      RSP := 'OK: alta de central de producci realitzada correctament. Id = ' || v_seq;

      INSERT INTO LOGS (NOMPROC,
                        LOGDATE,
                        PARIN,
                        PAROUT)
           VALUES ('pkg_abm.prc_alta_central_produccio',
                   SYSDATE,
                   v_parin,
                   'RSP = ' || RSP);

      COMMIT;
   EXCEPTION
      -- Si no se pot fer l'insert, se genera un registre de error a la taula LOGS
      WHEN err_null
      THEN
         RSP := 'ERROR: hi ha camps obligatoris sense informar';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_alta_central_produccio',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN err_no_tipus
      THEN
         RSP := 'ERROR: El tipus de central no existeix a la base de dades';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_alta_central_produccio',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN err_no_loc
      THEN
         RSP := 'ERROR: La localitat no existeix a la base de dades';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_alta_central_produccio',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN OTHERS
      THEN
         err_num := SQLCODE;
         err_msg := SUBSTR (SQLERRM, 1, 200);

         RSP := 'Error no tractat: ' || TO_CHAR (err_num) || ': ' || err_msg;

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_alta_central_produccio',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
   END prc_alta_central_produccio;

   PROCEDURE prc_baixa_central_produccio (
      p_idcentralprod   IN     central_produccio.idcentralprod%TYPE,
      RSP                  OUT VARCHAR2)
   IS
      /*******************************************************************************************
          NOM:        prc_baixa_central_produccio
          PROPSIT:   Dona de baixa una central de producci
          REQUISITS:  - S'ha d'informar l'Id de la central de producci
                      - El registre ha d'existir a la base de dades
                      - El registre no ha d'estar donat de baixa
          PARMETRES:
              IN:   Obligatoris:  p_idcentralprod: Identificador de la central de producci
              OUT:  RSP
          PRE-CONDICI:  Existeix el la central de producci que se vol donar de baixa
          POST-CONDICI: se dona de baixa la central de producci
          RETORNA:
              tot ha anat be:
                  RSP = 'OK: baixa de central de producci realitzada correctament'
              missatge de errors:
                  'ERROR: hi ha camps obligatoris sense informar'
                  'ERROR: La central de producci no existeix'
                  'ERROR: El registre ja est donat de baixa'
                  'ERROR: Error no tractat'
      *******************************************************************************************/
      err_null            EXCEPTION;
      err_no_idcentprod   EXCEPTION;
      err_prod_baixa      EXCEPTION;

      v_count             VARCHAR2 (1);
      v_parin             VARCHAR2 (300);
      err_num             NUMBER;
      err_msg             VARCHAR2 (100);
   BEGIN
      v_parin := 'p_idcentralprod: ' || p_idcentralprod;

      -- Verificaci de requisits
      IF (TRIM (p_idcentralprod) IS NULL)
      THEN
         RAISE err_null;
      END IF;

      -- Existeix Id a central_produccio
      BEGIN
         v_count := NULL;

         SELECT 1
           INTO v_count
           FROM central_produccio
          WHERE idcentralprod = p_idcentralprod AND ROWNUM = 1;
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            RAISE err_no_idcentprod;
      END;

      -- El registre no est donat de baixa
      BEGIN
         v_count := NULL;

         SELECT 1
           INTO v_count
           FROM central_produccio
          WHERE idcentralprod = p_idcentralprod AND data_baixa IS NOT NULL AND ROWNUM = 1;

         RAISE err_prod_baixa;
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            NULL;
      END;

      -- Si compleix els requisits, s'afegeix la data de baixa
      UPDATE CENTRAL_PRODUCCIO
         SET DATA_BAIXA = SYSDATE
       WHERE idcentralprod = p_idcentralprod;

      RSP := 'OK: baixa de central de producci realitzada correctament.';

      INSERT INTO LOGS (NOMPROC,
                        LOGDATE,
                        PARIN,
                        PAROUT)
           VALUES ('pkg_abm.prc_baixa_central_produccio',
                   SYSDATE,
                   v_parin,
                   'RSP = ' || RSP);

      COMMIT;
   EXCEPTION
      -- Si no se pot fer l'insert, se genera un registre de error a la taula LOGS
      WHEN err_null
      THEN
         RSP := 'ERROR: hi ha camps obligatoris sense informar';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_baixa_central_produccio',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN err_no_idcentprod
      THEN
         RSP := 'ERROR: La central de producci no existeix';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_baixa_central_produccio',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN err_prod_baixa
      THEN
         RSP := 'ERROR: El registre ja est donat de baixa';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_baixa_central_produccio',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN OTHERS
      THEN
         err_num := SQLCODE;
         err_msg := SUBSTR (SQLERRM, 1, 200);

         RSP := 'Error no tractat: ' || TO_CHAR (err_num) || ': ' || err_msg;

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_baixa_central_produccio',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
   END prc_baixa_central_produccio;

   PROCEDURE prc_modif_central_produccio (
      p_idcentralprod   IN     central_produccio.idcentralprod%TYPE,
      p_idtipusprod     IN     central_produccio.idtipusprod%TYPE,
      p_direccio        IN     central_produccio.direccio%TYPE,
      p_idlocalitat     IN     central_produccio.idlocalitat%TYPE,
      p_data_it         IN     central_produccio.data_it%TYPE,
      p_atribut_1       IN     central_produccio.atribut_1%TYPE,
      p_atribut_2       IN     central_produccio.atribut_2%TYPE,
      RSP                  OUT VARCHAR2)
   IS
      /*******************************************************************************************
          NOM:        prc_modif_central_produccio
          PROPSIT:   Modifica una central de producci
          REQUISITS:  - S'ha d'informar l'Id de la central de producci
                      - El registre ha d'existir a la base de dades
                      - Existeix el tipus de central de producci que se vol modificar
                      - Existeix la lnia de comunicaci que se vol modificar
                      - Existeix la localitat que se vol modificar
          PARMETRES:
              IN:   Obligatoris:  p_idcentralprod: Identificador de la central de producci
                    Opcionals:    p_idtipusprod: Identificador del tipus de central de producci
                                  p_direccio: Adrea de la central de producci
                                  p_idlocalitat: Identificador de la localitat de la direcci
                                  p_data_IT: Data de la darrera Inspecci Tcnica (IT)
                                  p_atribut_1: Aquest valor vindr definit depenent del tipus
                                  (nuclear, trmica...)
                                  p_atribut_2: En cas de ser una central de tipus nuclear, definim
                                  l'atribut 2 com els Kg de rebuig radioactiu generat.
              OUT:  RSP
          PRE-CONDICI:  Existeix la central de producci que se vol modificar
          POST-CONDICI: se modifica la central de producci
          RETORNA:
              tot ha anat be:
                  RSP = 'OK: modificaci de central de producci realitzada correctament'
              missatge de errors:
                  'ERROR: hi ha camps obligatoris sense informar'
                  'ERROR: La central de producci no existeix'
                  'ERROR: El tipus de central no existeix a la base de dades'
                  'ERROR: La localitat no existeix a la base de dades'
                  'ERROR: Error no tractat'
      *******************************************************************************************/
      err_null            EXCEPTION;
      err_no_idcentprod   EXCEPTION;
      err_no_tipus        EXCEPTION;
      err_no_loc          EXCEPTION;

      v_count             VARCHAR2 (1);
      v_parin             VARCHAR2 (300);
      err_num             NUMBER;
      err_msg             VARCHAR2 (100);
   BEGIN
      v_parin :=
            'p_idcentralprod: '
         || p_idcentralprod
         || ', p_idtipusprod: '
         || p_idtipusprod
         || ', p_direccio: '
         || p_direccio
         || ', p_idlocalitat: '
         || p_idlocalitat
         || ', p_data_it: '
         || p_data_it
         || ', p_atribut_1: '
         || p_atribut_1
         || ', p_atribut_2: '
         || p_atribut_2;

      -- Verificaci de requisits
      -- Han d'estar informats tots els camps obligatoris (NOT NULL)
      IF (TRIM (p_idcentralprod) IS NULL)
      THEN
         RAISE err_null;
      END IF;

      -- Existeix Id a central_produccio
      BEGIN
         v_count := NULL;

         SELECT 1
           INTO v_count
           FROM central_produccio
          WHERE idcentralprod = p_idcentralprod AND ROWNUM = 1;
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            RAISE err_no_idcentprod;
      END;

      -- Existeix Id a tipus_centralprod
      IF p_idtipusprod IS NOT NULL
      THEN
         BEGIN
            v_count := NULL;

            SELECT 1
              INTO v_count
              FROM tipus_centralprod
             WHERE idtipusprod = p_idtipusprod AND ROWNUM = 1;
         EXCEPTION
            WHEN NO_DATA_FOUND
            THEN
               RAISE err_no_tipus;
         END;
      END IF;

      -- Existeix Id a localitat
      IF p_idlocalitat IS NOT NULL
      THEN
         BEGIN
            v_count := NULL;

            SELECT 1
              INTO v_count
              FROM localitat
             WHERE idlocalitat = p_idlocalitat AND ROWNUM = 1;
         EXCEPTION
            WHEN NO_DATA_FOUND
            THEN
               RAISE err_no_loc;
         END;
      END IF;

      -- Si compleix els requisits, se fa la modiicaci del registre
      UPDATE CENTRAL_PRODUCCIO
         SET IDTIPUSPROD = NVL (p_idtipusprod, IDTIPUSPROD),
             DIRECCIO = NVL (p_direccio, DIRECCIO),
             IDLOCALITAT = NVL (p_idlocalitat, IDLOCALITAT),
             DATA_IT = NVL (p_data_it, DATA_IT),
             ATRIBUT_1 = NVL (p_atribut_1, ATRIBUT_1),
             ATRIBUT_2 = NVL (p_atribut_1, ATRIBUT_2),
             DATA_LAST_MOD = SYSDATE
       WHERE IDCENTRALPROD = p_idcentralprod;

      RSP := 'OK: modificaci de central de producci realitzada correctament.';

      INSERT INTO LOGS (NOMPROC,
                        LOGDATE,
                        PARIN,
                        PAROUT)
           VALUES ('pkg_abm.prc_modif_central_produccio',
                   SYSDATE,
                   v_parin,
                   'RSP = ' || RSP);

      COMMIT;
   EXCEPTION
      -- Si no se pot fer l'insert, se genera un registre de error a la taula LOGS
      WHEN err_null
      THEN
         RSP := 'ERROR: hi ha camps obligatoris sense informar';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_modif_central_produccio',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN err_no_idcentprod
      THEN
         RSP := 'ERROR: La central de producci no existeix';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_modif_central_produccio',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN err_no_tipus
      THEN
         RSP := 'ERROR: El tipus de central no existeix a la base de dades';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_modif_central_produccio',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN err_no_loc
      THEN
         RSP := 'ERROR: La localitat no existeix a la base de dades';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_modif_central_produccio',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN OTHERS
      THEN
         err_num := SQLCODE;
         err_msg := SUBSTR (SQLERRM, 1, 200);

         RSP := 'Error no tractat: ' || TO_CHAR (err_num) || ': ' || err_msg;

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_modif_central_produccio',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
   END prc_modif_central_produccio;

   PROCEDURE prc_react_central_produccio (
      p_idcentralprod   IN     central_produccio.idcentralprod%TYPE,
      RSP                  OUT VARCHAR2)
   IS
      /*******************************************************************************************
          NOM:        prc_react_central_produccio
          PROPSIT:   Reactiva una central de producci donada de baixa
          REQUISITS:  - S'ha d'informar l'Id de la central de producci
                      - El registre ha d'existir a la base de dades
          PARMETRES:
              IN:   Obligatoris:  p_idcentralprod: Identificador de la central de producci
              OUT:  RSP
          PRE-CONDICI:  Existeix la central de producci que se vol reactivar
          POST-CONDICI: se reactiva la central de producci
          RETORNA:
              tot ha anat be:
                  RSP = 'OK: reactivaci de central de producci realitzada correctament'
              missatge de errors:
                  'ERROR: hi ha camps obligatoris sense informar'
                  'ERROR: La central de producci no existeix'
                  'ERROR: El registre ja est donat d'alta'
                  'ERROR: Error no tractat'
      *******************************************************************************************/
      err_null            EXCEPTION;
      err_no_idcentprod   EXCEPTION;
      err_prod_alta       EXCEPTION;

      v_count             VARCHAR2 (1);
      v_parin             VARCHAR2 (300);
      err_num             NUMBER;
      err_msg             VARCHAR2 (100);
   BEGIN
      v_parin := 'p_idcentralprod: ' || p_idcentralprod;

      -- Verificaci de requisits
      IF (TRIM (p_idcentralprod) IS NULL)
      THEN
         RAISE err_null;
      END IF;

      -- Existeix Id a central_produccio
      BEGIN
         v_count := NULL;

         SELECT 1
           INTO v_count
           FROM central_produccio
          WHERE idcentralprod = p_idcentralprod AND ROWNUM = 1;
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            RAISE err_no_idcentprod;
      END;

      -- El registre est donat de baixa
      BEGIN
         v_count := NULL;

         SELECT 1
           INTO v_count
           FROM central_produccio
          WHERE idcentralprod = p_idcentralprod AND data_baixa IS NULL AND ROWNUM = 1;

         RAISE err_prod_alta;
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            NULL;
      END;

      -- Si compleix els requisits, s'elimina la data de baixa
      UPDATE CENTRAL_PRODUCCIO
         SET DATA_BAIXA = NULL
       WHERE idcentralprod = p_idcentralprod;

      RSP := 'OK: reactivaci de central de producci realitzada correctament.';

      INSERT INTO LOGS (NOMPROC,
                        LOGDATE,
                        PARIN,
                        PAROUT)
           VALUES ('pkg_abm.prc_react_central_produccio',
                   SYSDATE,
                   v_parin,
                   'RSP = ' || RSP);

      COMMIT;
   EXCEPTION
      -- Si no se pot fer l'insert, se genera un registre de error a la taula LOGS
      WHEN err_null
      THEN
         RSP := 'ERROR: hi ha camps obligatoris sense informar';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_react_central_produccio',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN err_no_idcentprod
      THEN
         RSP := 'ERROR: La central de producci no existeix';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_react_central_produccio',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN err_prod_alta
      THEN
         RSP := 'ERROR: El registre ja est donat d''alta';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_react_central_produccio',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN OTHERS
      THEN
         err_num := SQLCODE;
         err_msg := SUBSTR (SQLERRM, 1, 200);

         RSP := 'Error no tractat: ' || TO_CHAR (err_num) || ': ' || err_msg;

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_react_central_produccio',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
   END prc_react_central_produccio;

   /******************************** Central_distribucio **************************************/

   PROCEDURE prc_alta_central_distribucio (
      p_idlinia       IN     central_distribucio.idlinia%TYPE,
      p_direccio      IN     central_distribucio.direccio%TYPE,
      p_idlocalitat   IN     central_distribucio.idlocalitat%TYPE,
      p_energia_max   IN     central_distribucio.energia_max%TYPE,
      RSP                OUT VARCHAR2)
   IS
      /*******************************************************************************************
          NOM:        prc_alta_central_distribucio
          PROPSIT:   Dona d'alta una central de distribuci
          REQUISITS:  - Han d'estar informats tots els camps obligatoris (NOT NULL)
                      - El registre no ha d'existir
                      - Els Id's de central de producci i localitat han d'existir a la base de dades
          PARMETRES:
              IN:   Obligatoris:  p_idlinia: Identificador de la linia de comunicaci
                                  p_direccio: Adrea de la central de producci
                                  p_idlocalitat: Identificador de la localitat de la direcci
                                  p_energia_max: Energia mxima produda (KWh)
              OUT:  RSP
          PRE-CONDICI:  No existeix la central de distribuci
          POST-CONDICI: Se dona d'alta la central de distribuci
          RETORNA:
              tot ha anat be:
                  RSP = 'OK: alta de central de distribucio realitzada correctament. Id = NumId'
              missatge de errors:
                  'ERROR: hi ha camps obligatoris sense informar'
                  'ERROR: La linia de comunicaci no existeix a la base de dades'
                  'ERROR: La central de producci no existeix a la base de dades'
                  'ERROR: La localitat no existeix a la base de dades'
                  'ERROR: Error no tractat'
      *******************************************************************************************/
      err_null            EXCEPTION;
      err_no_idcentprod   EXCEPTION;
      err_no_linia        EXCEPTION;
      err_no_loc          EXCEPTION;

      v_count             VARCHAR2 (1);
      v_parin             VARCHAR2 (300);
      v_idcentralprod     central_distribucio.idcentralprod%TYPE;
      v_seq               NUMBER;
      err_num             NUMBER;
      err_msg             VARCHAR2 (100);
   BEGIN
      v_parin :=
            'p_idlinia: '
         || p_idlinia
         || ', p_direccio: '
         || p_direccio
         || ', p_idlocalitat: '
         || p_idlocalitat
         || ', p_energia_max: '
         || p_energia_max;

      -- Verificaci de requisits
      -- Han d'estar informats tots els camps obligatoris (NOT NULL)
      IF (   TRIM (p_idlinia) IS NULL
          OR TRIM (p_direccio) IS NULL
          OR TRIM (p_idlocalitat) IS NULL
          OR TRIM (p_energia_max) IS NULL)
      THEN
         RAISE err_null;
      END IF;

      -- Existeix Id a linia_comunicacio
      BEGIN
         v_count := NULL;

         SELECT 1
           INTO v_count
           FROM linia_comunicacio
          WHERE idlinia = p_idlinia AND ROWNUM = 1;
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            RAISE err_no_linia;
      END;

      -- Existeix Id a localitat
      BEGIN
         v_count := NULL;

         SELECT 1
           INTO v_count
           FROM localitat
          WHERE idlocalitat = p_idlocalitat AND ROWNUM = 1;
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            RAISE err_no_loc;
      END;

      -- Agafarem Id de central_produccio amb l'id de linia
      BEGIN
         SELECT idcentralprod
           INTO v_idcentralprod
           FROM linia_comunicacio
          WHERE idlinia = p_idlinia AND ROWNUM = 1;
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            RAISE err_no_idcentprod;
      END;

      -- Si compleix els requisits, se fa la inserci
      INSERT INTO CENTRAL_DISTRIBUCIO (IDCENTRALPROD,
                                       IDLINIA,
                                       DIRECCIO,
                                       IDLOCALITAT,
                                       ENERGIA_MAX,
                                       DATA_ALTA)
           VALUES (v_idcentralprod,
                   p_idlinia,
                   p_direccio,
                   p_idlocalitat,
                   p_energia_max,
                   SYSDATE);

      SELECT seq_CENTRAL_DISTRIBUCIO.CURRVAL INTO v_seq FROM DUAL;

      RSP := 'OK: alta de central de distribuci realitzada correctament. Id = ' || v_seq;

      INSERT INTO LOGS (NOMPROC,
                        LOGDATE,
                        PARIN,
                        PAROUT)
           VALUES ('pkg_abm.prc_alta_central_distribucio',
                   SYSDATE,
                   v_parin,
                   'RSP = ' || RSP);

      COMMIT;
   EXCEPTION
      -- Si no se pot fer l'insert, se genera un registre de error a la taula LOGS
      WHEN err_null
      THEN
         RSP := 'ERROR: hi ha camps obligatoris sense informar';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_alta_central_distribucio',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN err_no_idcentprod
      THEN
         RSP := 'ERROR: La central de producci no existeix a la base de dades';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_alta_central_distribucio',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN err_no_linia
      THEN
         RSP := 'ERROR: La linia de comunicaci no existeix a la base de dades';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_alta_central_distribucio',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN err_no_loc
      THEN
         RSP := 'ERROR: La localitat no existeix a la base de dades';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_alta_central_distribucio',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN OTHERS
      THEN
         err_num := SQLCODE;
         err_msg := SUBSTR (SQLERRM, 1, 200);

         RSP := 'Error no tractat: ' || TO_CHAR (err_num) || ': ' || err_msg;

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_alta_central_distribucio',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
   END prc_alta_central_distribucio;

   PROCEDURE prc_baixa_central_distribucio (
      p_idcentraldis   IN     central_distribucio.idcentraldis%TYPE,
      RSP                 OUT VARCHAR2)
   IS
      /*******************************************************************************************
          NOM:        prc_baixa_central_distribucio
          PROPSIT:   Dona de baixa una central de distribucio
          REQUISITS:  - S'ha d'informar l'Id de la central de distribucio
                      - El registre ha d'existir a la base de dades
                      - El registre no ha d'estar donat de baixa
          PARMETRES:
              IN:   Obligatoris:  p_idcentraldis: Identificador de la central de distribucio
              OUT:  RSP
          PRE-CONDICI:  Existeix el la central de distribucio que se vol donar de baixa
          POST-CONDICI: se dona de baixa la central de distribucio
          RETORNA:
              tot ha anat be:
                  RSP = 'OK: baixa de central de distribucio realitzada correctament'
              missatge de errors:
                  'ERROR: hi ha camps obligatoris sense informar'
                  'ERROR: La central de distribucio no existeix'
                  'ERROR: El registre ja est donat de baixa'
                  'ERROR: Error no tractat'
      *******************************************************************************************/
      err_null           EXCEPTION;
      err_no_idcentdis   EXCEPTION;
      err_dis_baixa      EXCEPTION;

      v_count            VARCHAR2 (1);
      v_parin            VARCHAR2 (300);
      err_num            NUMBER;
      err_msg            VARCHAR2 (100);
   BEGIN
      v_parin := 'p_idcentraldis: ' || p_idcentraldis;

      -- Verificaci de requisits
      IF (TRIM (p_idcentraldis) IS NULL)
      THEN
         RAISE err_null;
      END IF;

      -- Existeix Id a central_distribucio
      BEGIN
         v_count := NULL;

         SELECT 1
           INTO v_count
           FROM central_distribucio
          WHERE idcentraldis = p_idcentraldis AND ROWNUM = 1;
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            RAISE err_no_idcentdis;
      END;

      -- El registre no est donat de baixa
      BEGIN
         v_count := NULL;

         SELECT 1
           INTO v_count
           FROM central_distribucio
          WHERE idcentraldis = p_idcentraldis AND data_baixa IS NOT NULL AND ROWNUM = 1;

         RAISE err_dis_baixa;
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            NULL;
      END;

      -- Si compleix els requisits, s'afegeix la data de baixa
      UPDATE CENTRAL_DISTRIBUCIO
         SET DATA_BAIXA = SYSDATE
       WHERE idcentraldis = p_idcentraldis;

      RSP := 'OK: baixa de central de distribuci realitzada correctament.';

      INSERT INTO LOGS (NOMPROC,
                        LOGDATE,
                        PARIN,
                        PAROUT)
           VALUES ('pkg_abm.prc_baixa_central_distribucio',
                   SYSDATE,
                   v_parin,
                   'RSP = ' || RSP);

      COMMIT;
   EXCEPTION
      -- Si no se pot fer l'insert, se genera un registre de error a la taula LOGS
      WHEN err_null
      THEN
         RSP := 'ERROR: hi ha camps obligatoris sense informar';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_baixa_central_distribucio',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN err_no_idcentdis
      THEN
         RSP := 'ERROR: La central de distribucio no existeix';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_baixa_central_distribucio',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN err_dis_baixa
      THEN
         RSP := 'ERROR: El registre ja est donat de baixa';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_baixa_central_distribucio',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN OTHERS
      THEN
         err_num := SQLCODE;
         err_msg := SUBSTR (SQLERRM, 1, 200);

         RSP := 'Error no tractat: ' || TO_CHAR (err_num) || ': ' || err_msg;

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_baixa_central_distribucio',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
   END prc_baixa_central_distribucio;

   PROCEDURE prc_modif_central_distribucio (
      p_idcentraldis   IN     central_distribucio.idcentraldis%TYPE,
      p_idlinia        IN     central_distribucio.idlinia%TYPE,
      p_direccio       IN     central_distribucio.direccio%TYPE,
      p_idlocalitat    IN     central_distribucio.idlocalitat%TYPE,
      p_energia_max    IN     central_distribucio.energia_max%TYPE,
      RSP                 OUT VARCHAR2)
   IS
      /*******************************************************************************************
          NOM:        prc_modif_central_distribuci
          PROPSIT:   Modifica una central de distribuci
          REQUISITS:  - S'ha d'informar l'Id de la central de distribuci
                      - El registre ha d'existir a la base de dades
                      - Existeix la central de producci que se vol modificar
                      - Existeix la localitat que se vol modificar
          PARMETRES:
              IN:   Obligatoris:  p_idcentraldis: Identificador de la central de distribuci
                    Opcionals:    p_idcentralprod: Identificador de la central de producci
                                  p_idlinia: Identificador de la linia de comunicaci
                                  p_direccio: Adrea de la central de producci
                                  p_idlocalitat: Identificador de la localitat de la direcci
                                  p_energia_max: Energia mxima produda (KWh)
              OUT:  RSP
          PRE-CONDICI:  Existeix la central de distribuci que se vol modificar
          POST-CONDICI: se modifica la central de distribuci
          RETORNA:
              tot ha anat be:
                  RSP = 'OK: modificaci de central de distribuci realitzada correctament'
              missatge de errors:
                  'ERROR: hi ha camps obligatoris sense informar'
                  'ERROR: La central de distribuci no existeix'
                  'ERROR: La central de producci no existeix a la base de dades'
                  'ERROR: La linia de comunicaci no existeix a la base de dades'
                  'ERROR: La localitat no existeix a la base de dades'
                  'ERROR: Error no tractat'
      *******************************************************************************************/
      err_null            EXCEPTION;
      err_no_idcentdis    EXCEPTION;
      err_no_idcentprod   EXCEPTION;
      err_no_linia        EXCEPTION;
      err_no_loc          EXCEPTION;

      v_count             VARCHAR2 (1);
      v_parin             VARCHAR2 (300);
      v_idcentralprod     central_distribucio.idcentralprod%TYPE;
      err_num             NUMBER;
      err_msg             VARCHAR2 (100);
   BEGIN
      v_parin :=
            'p_idcentraldis: '
         || p_idcentraldis
         || ', p_idlinia: '
         || p_idlinia
         || ', p_direccio: '
         || p_direccio
         || ', p_idlocalitat: '
         || p_idlocalitat
         || ', p_energia_max: '
         || p_energia_max;

      -- Verificaci de requisits
      -- Han d'estar informats tots els camps obligatoris (NOT NULL)
      IF (TRIM (p_idcentraldis) IS NULL)
      THEN
         RAISE err_null;
      END IF;

      -- Existeix Id a central_distribucio
      BEGIN
         v_count := NULL;

         SELECT 1
           INTO v_count
           FROM central_distribucio
          WHERE idcentraldis = p_idcentraldis AND ROWNUM = 1;
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            RAISE err_no_idcentdis;
      END;

      -- Existeix Id a linia_comunicacio
      IF p_idlinia IS NOT NULL
      THEN
         BEGIN
            v_count := NULL;

            SELECT 1
              INTO v_count
              FROM linia_comunicacio
             WHERE idlinia = p_idlinia AND ROWNUM = 1;
         EXCEPTION
            WHEN NO_DATA_FOUND
            THEN
               RAISE err_no_linia;
         END;
      END IF;

      -- Existeix Id a localitat
      IF p_idlocalitat IS NOT NULL
      THEN
         BEGIN
            v_count := NULL;

            SELECT 1
              INTO v_count
              FROM localitat
             WHERE idlocalitat = p_idlocalitat AND ROWNUM = 1;
         EXCEPTION
            WHEN NO_DATA_FOUND
            THEN
               RAISE err_no_loc;
         END;
      END IF;

      -- Agafarem Id de central_produccio amb l'id de linia
      BEGIN
         SELECT idcentralprod
           INTO v_idcentralprod
           FROM linia_comunicacio
          WHERE idlinia = NVL (p_idlinia, idlinia) AND ROWNUM = 1;
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            RAISE err_no_idcentprod;
      END;

      -- Si compleix els requisits, se fa la inserci
      UPDATE CENTRAL_DISTRIBUCIO
         SET IDCENTRALPROD = NVL (v_idcentralprod, IDCENTRALPROD),
             DIRECCIO = NVL (p_direccio, DIRECCIO),
             IDLOCALITAT = NVL (p_idlocalitat, IDLOCALITAT),
             ENERGIA_MAX = NVL (p_energia_max, ENERGIA_MAX),
             DATA_LAST_MOD = SYSDATE
       WHERE IDCENTRALDIS = p_idcentraldis;

      RSP := 'OK: modificaci de central de distribuci realitzada correctament.';

      INSERT INTO LOGS (NOMPROC,
                        LOGDATE,
                        PARIN,
                        PAROUT)
           VALUES ('pkg_abm.prc_modif_central_distribucio',
                   SYSDATE,
                   v_parin,
                   'RSP = ' || RSP);

      COMMIT;
   EXCEPTION
      -- Si no se pot fer l'insert, se genera un registre de error a la taula LOGS
      WHEN err_null
      THEN
         RSP := 'ERROR: hi ha camps obligatoris sense informar';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_modif_central_distribucio',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN err_no_idcentdis
      THEN
         RSP := 'ERROR: La central de distribuci no existeix';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_modif_central_distribucio',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN err_no_idcentprod
      THEN
         RSP := 'ERROR: La central de producci no existeix a la base de dades';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_modif_central_distribucio',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN err_no_linia
      THEN
         RSP := 'ERROR: La linia de comunicaci no existeix a la base de dades';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_modif_central_produccio',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN err_no_loc
      THEN
         RSP := 'ERROR: La localitat no existeix a la base de dades';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_modif_central_distribucio',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN OTHERS
      THEN
         err_num := SQLCODE;
         err_msg := SUBSTR (SQLERRM, 1, 200);

         RSP := 'Error no tractat: ' || TO_CHAR (err_num) || ': ' || err_msg;

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_modif_central_distribucio',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
   END prc_modif_central_distribucio;

   PROCEDURE prc_react_central_distribucio (
      p_idcentraldis   IN     central_distribucio.idcentraldis%TYPE,
      RSP                 OUT VARCHAR2)
   IS
      /*******************************************************************************************
          NOM:        prc_react_central_distribucio
          PROPSIT:   Reactiva una central de distribucio
          REQUISITS:  - S'ha d'informar l'Id de la central de distribucio
                      - El registre ha d'existir a la base de dades
          PARMETRES:
              IN:   Obligatoris:  p_idcentraldis: Identificador de la central de distribucio
              OUT:  RSP
          PRE-CONDICI:  Existeix el la central de distribucio que se vol reactivar
          POST-CONDICI: se reactiva la central de distribucio
          RETORNA:
              tot ha anat be:
                  RSP = 'OK: reactivaci de central de distribucio realitzada correctament'
              missatge de errors:
                  'ERROR: hi ha camps obligatoris sense informar'
                  'ERROR: La central de distribucio no existeix'
                  'ERROR: El registre ja est donat d'alta'
                  'ERROR: Error no tractat'
      *******************************************************************************************/
      err_null           EXCEPTION;
      err_no_idcentdis   EXCEPTION;
      err_dis_alta       EXCEPTION;

      v_count            VARCHAR2 (1);
      v_parin            VARCHAR2 (300);
      err_num            NUMBER;
      err_msg            VARCHAR2 (100);
   BEGIN
      v_parin := 'p_idcentraldis: ' || p_idcentraldis;

      -- Verificaci de requisits
      IF (TRIM (p_idcentraldis) IS NULL)
      THEN
         RAISE err_null;
      END IF;

      -- Existeix Id a central_distribucio
      BEGIN
         v_count := NULL;

         SELECT 1
           INTO v_count
           FROM central_distribucio
          WHERE idcentraldis = p_idcentraldis AND ROWNUM = 1;
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            RAISE err_no_idcentdis;
      END;

      -- El registre no est donat d'alta
      BEGIN
         v_count := NULL;

         SELECT 1
           INTO v_count
           FROM central_distribucio
          WHERE idcentraldis = p_idcentraldis AND data_baixa IS NULL AND ROWNUM = 1;

         RAISE err_dis_alta;
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            NULL;
      END;

      -- Si compleix els requisits, s'elimina la data de baixa
      UPDATE CENTRAL_DISTRIBUCIO
         SET DATA_BAIXA = NULL
       WHERE idcentraldis = p_idcentraldis;

      RSP := 'OK: reactivaci de central de distribuci realitzada correctament.';

      INSERT INTO LOGS (NOMPROC,
                        LOGDATE,
                        PARIN,
                        PAROUT)
           VALUES ('pkg_abm.prc_react_central_distribucio',
                   SYSDATE,
                   v_parin,
                   'RSP = ' || RSP);

      COMMIT;
   EXCEPTION
      -- Si no se pot fer l'insert, se genera un registre de error a la taula LOGS
      WHEN err_null
      THEN
         RSP := 'ERROR: hi ha camps obligatoris sense informar';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_react_central_distribucio',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN err_no_idcentdis
      THEN
         RSP := 'ERROR: La central de distribucio no existeix';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_react_central_distribucio',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN err_dis_alta
      THEN
         RSP := 'ERROR: El registre ja est donat d''alta';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_react_central_distribucio',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN OTHERS
      THEN
         err_num := SQLCODE;
         err_msg := SUBSTR (SQLERRM, 1, 200);

         RSP := 'Error no tractat: ' || TO_CHAR (err_num) || ': ' || err_msg;

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_react_central_distribucio',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
   END prc_react_central_distribucio;

   /************************************** Comptador *****************************************/

   PROCEDURE prc_alta_comptador (p_nif           IN     comptador.nif%TYPE,
                                 p_potencia      IN     comptador.potencia%TYPE,
                                 p_direccio      IN     comptador.direccio%TYPE,
                                 p_idlocalitat   IN     comptador.idlocalitat%TYPE,
                                 p_idmodel       IN     comptador.idmodel%TYPE,
                                 RSP                OUT VARCHAR2)
   IS
      /*******************************************************************************************
          NOM:        prc_alta_comptador
          PROPSIT:   Dona d'alta un comptador
          REQUISITS:  - Han d'estar informats tots els camps obligatoris (NOT NULL)
                      - El registre no ha d'existir
                      - Els Id's de central distribuci, NIF (client), localitat i model han d'existir a la base de dades
          PARMETRES:
              IN:   Obligatoris:  p_nif: NIF del client
                                  p_potencia: potncia contractada pels clients
                                  p_direccio: Adrea del comptador
                                  p_idlocalitat: Identificador de la localitat de la direcci
                                  p_idmodel: Identificador del model del comptador
              OUT:  RSP
          PRE-CONDICI:  No existeix el comptador
          POST-CONDICI: Se dona d'alta el comptador
          RETORNA:
              tot ha anat be:
                  RSP = 'OK: alta de comptador realitzada correctament. Id = NumId'
              missatge de errors:
                  'ERROR: hi ha camps obligatoris sense informar'
                  'ERROR: El comptador ja existeix'
                  'ERROR: El client no existeix a la base de dades'
                  'ERROR: La localitat no existeix a la base de dades'
                  'ERROR: Error no tractat'
      *******************************************************************************************/
      err_null        EXCEPTION;
      err_no_client   EXCEPTION;
      err_no_loc      EXCEPTION;

      v_count         VARCHAR2 (1);
      v_parin         VARCHAR2 (300);
      v_seq           NUMBER;
      err_num         NUMBER;
      err_msg         VARCHAR2 (100);
   BEGIN
      v_parin :=
            ', p_nif: '
         || p_nif
         || ', p_potencia: '
         || p_potencia
         || ', p_direccio: '
         || p_direccio
         || ', p_idlocalitat: '
         || p_idlocalitat
         || ', p_idmodel: '
         || p_idmodel;

      -- Verificaci de requisits
      -- Han d'estar informats tots els camps obligatoris (NOT NULL)
      IF (   TRIM (p_nif) IS NULL
          OR TRIM (p_potencia) IS NULL
          OR TRIM (p_direccio) IS NULL
          OR TRIM (p_idlocalitat) IS NULL
          OR TRIM (p_idmodel) IS NULL)
      THEN
         RAISE err_null;
      END IF;

      -- Existeix Id a client
      BEGIN
         v_count := NULL;

         SELECT 1
           INTO v_count
           FROM client
          WHERE nif = p_nif AND ROWNUM = 1;
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            RAISE err_no_client;
      END;

      -- Existeix Id a localitat
      BEGIN
         v_count := NULL;

         SELECT 1
           INTO v_count
           FROM localitat
          WHERE idlocalitat = p_idlocalitat AND ROWNUM = 1;
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            RAISE err_no_loc;
      END;

      -- Si compleix els requisits, se fa la inserci
      INSERT INTO COMPTADOR (NIF,
                             POTENCIA,
                             DIRECCIO,
                             IDLOCALITAT,
                             IDMODEL,
                             DATA_ALTA)
           VALUES (p_nif,
                   p_potencia,
                   p_direccio,
                   p_idlocalitat,
                   p_idmodel,
                   SYSDATE);

      SELECT seq_COMPTADOR.CURRVAL INTO v_seq FROM DUAL;

      RSP := 'OK: alta de comptador realitzada correctament. Id = ' || v_seq;

      INSERT INTO LOGS (NOMPROC,
                        LOGDATE,
                        PARIN,
                        PAROUT)
           VALUES ('pkg_abm.prc_alta_comptador',
                   SYSDATE,
                   v_parin,
                   'RSP = ' || RSP);

      COMMIT;
   EXCEPTION
      -- Si no se pot fer l'insert, se genera un registre de error a la taula LOGS
      WHEN err_null
      THEN
         RSP := 'ERROR: hi ha camps obligatoris sense informar';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_alta_comptador',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN err_no_client
      THEN
         RSP := 'ERROR: El client no existeix a la base de dades';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_alta_comptador',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN err_no_loc
      THEN
         RSP := 'ERROR: La localitat no existeix a la base de dades';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_alta_comptador',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN OTHERS
      THEN
         err_num := SQLCODE;
         err_msg := SUBSTR (SQLERRM, 1, 200);

         RSP := 'Error no tractat: ' || TO_CHAR (err_num) || ': ' || err_msg;

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_alta_comptador',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
   END prc_alta_comptador;

   PROCEDURE prc_baixa_comptador (p_idcontracte IN comptador.idcontracte%TYPE, RSP OUT VARCHAR2)
   IS
      /*******************************************************************************************
          NOM:        prc_baixa_comptador
          PROPSIT:   Dona de baixa un comptador
          REQUISITS:  - S'ha d'informar l'Id del comptador
                      - El registre ha d'existir a la base de dades
                      - El registre no ha d'estar donat de baixa
          PARMETRES:
              IN:   Obligatoris:  p_idcontracte: Identificador del comptador
              OUT:  RSP
          PRE-CONDICI:  Existeix el comptador que se vol donar de baixa
          POST-CONDICI: se dona de baixa el comptador
          RETORNA:
              tot ha anat be:
                  RSP = 'OK: baixa de central de distribucio realitzada correctament'
              missatge de errors:
                  'ERROR: hi ha camps obligatoris sense informar'
                  'ERROR: El comptador no existeix'
                  'ERROR: El registre ja est donat de baixa'
                  'ERROR: Error no tractat'
      *******************************************************************************************/
      err_null             EXCEPTION;
      err_no_idcontracte   EXCEPTION;
      err_compt_baixa      EXCEPTION;

      v_count              VARCHAR2 (1);
      v_parin              VARCHAR2 (300);
      err_num              NUMBER;
      err_msg              VARCHAR2 (100);
   BEGIN
      v_parin := 'p_idcontracte: ' || p_idcontracte;

      -- Verificaci de requisits
      IF (TRIM (p_idcontracte) IS NULL)
      THEN
         RAISE err_null;
      END IF;

      -- Existeix Id a comptador
      BEGIN
         v_count := NULL;

         SELECT 1
           INTO v_count
           FROM comptador
          WHERE idcontracte = p_idcontracte AND ROWNUM = 1;
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            RAISE err_no_idcontracte;
      END;

      -- El registre no est donat de baixa
      BEGIN
         v_count := NULL;

         SELECT 1
           INTO v_count
           FROM comptador
          WHERE idcontracte = p_idcontracte AND data_baixa IS NOT NULL AND ROWNUM = 1;

         RAISE err_compt_baixa;
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            NULL;
      END;

      -- Si compleix els requisits, s'afegeix la data de baixa
      UPDATE COMPTADOR
         SET DATA_BAIXA = SYSDATE
       WHERE idcontracte = p_idcontracte;

      RSP := 'OK: baixa de comptador realitzada correctament.';

      INSERT INTO LOGS (NOMPROC,
                        LOGDATE,
                        PARIN,
                        PAROUT)
           VALUES ('pkg_abm.prc_baixa_comptador',
                   SYSDATE,
                   v_parin,
                   'RSP = ' || RSP);

      COMMIT;
   EXCEPTION
      -- Si no se pot fer l'insert, se genera un registre de error a la taula LOGS
      WHEN err_null
      THEN
         RSP := 'ERROR: hi ha camps obligatoris sense informar';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_baixa_comptador',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN err_no_idcontracte
      THEN
         RSP := 'ERROR: El comptador no existeix';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_baixa_comptador',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN err_compt_baixa
      THEN
         RSP := 'ERROR: El registre ja est donat de baixa';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_baixa_comptador',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN OTHERS
      THEN
         err_num := SQLCODE;
         err_msg := SUBSTR (SQLERRM, 1, 200);

         RSP := 'Error no tractat: ' || TO_CHAR (err_num) || ': ' || err_msg;

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_baixa_comptador',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
   END prc_baixa_comptador;

   PROCEDURE prc_modif_comptador (p_idcontracte   IN     comptador.idcontracte%TYPE,
                                  p_nif           IN     comptador.nif%TYPE,
                                  p_potencia      IN     comptador.potencia%TYPE,
                                  p_direccio      IN     comptador.direccio%TYPE,
                                  p_idlocalitat   IN     comptador.idlocalitat%TYPE,
                                  p_idmodel       IN     comptador.idmodel%TYPE,
                                  RSP                OUT VARCHAR2)
   IS
      /*******************************************************************************************
          NOM:        prc_modif_comptador
          PROPSIT:   Modifica un comptador
          REQUISITS:  - S'ha d'informar l'Id del comptador
                      - El registre ha d'existir a la base de dades
                      - Els Id's de NIF (client), localitat i model han d'existir a la base de dades
          PARMETRES:
              IN:   Obligatoris:  p_idcontracte: Identificador del comptador
                    Opcionals:    p_nif: NIF del client
                                  p_potencia: potncia contractada pels clients
                                  p_direccio: Adrea del comptador
                                  p_idlocalitat: Identificador de la localitat de la direcci
                                  p_idmodel: Identificador del model del comptador
              OUT:  RSP
          PRE-CONDICI:  Existeix el comptador que se vol modificar
          POST-CONDICI: Se modifica el comptador
          RETORNA:
              tot ha anat be:
                  RSP = 'OK: alta de comptador realitzada correctament. Id = NumId'
              missatge de errors:
                  'ERROR: hi ha camps obligatoris sense informar'
                  'ERROR: El comptador no existeix'
                  'ERROR: El client no existeix a la base de dades'
                  'ERROR: La localitat no existeix a la base de dades'
                  'ERROR: Error no tractat'
      *******************************************************************************************/
      err_null             EXCEPTION;
      err_no_idcontracte   EXCEPTION;
      err_no_client        EXCEPTION;
      err_no_loc           EXCEPTION;

      v_count              VARCHAR2 (1);
      v_parin              VARCHAR2 (300);
      err_num              NUMBER;
      err_msg              VARCHAR2 (100);
   BEGIN
      v_parin :=
            'p_idcontracte: '
         || p_idcontracte
         || ', p_nif: '
         || p_nif
         || ', p_potencia: '
         || p_potencia
         || ', p_direccio: '
         || p_direccio
         || ', p_idlocalitat: '
         || p_idlocalitat
         || ', p_idmodel: '
         || p_idmodel;

      -- Verificaci de requisits
      -- Han d'estar informats tots els camps obligatoris (NOT NULL)
      IF (TRIM (p_idcontracte) IS NULL)
      THEN
         RAISE err_null;
      END IF;

      -- Existeix Id a comptador
      BEGIN
         v_count := NULL;

         SELECT 1
           INTO v_count
           FROM comptador
          WHERE idcontracte = p_idcontracte AND ROWNUM = 1;
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            RAISE err_no_idcontracte;
      END;

      -- Existeix Id a client
      IF TRIM (p_nif) IS NOT NULL
      THEN
         BEGIN
            v_count := NULL;

            SELECT 1
              INTO v_count
              FROM client
             WHERE nif = p_nif AND ROWNUM = 1;
         EXCEPTION
            WHEN NO_DATA_FOUND
            THEN
               RAISE err_no_client;
         END;
      END IF;

      -- Existeix Id a localitat
      IF TRIM (p_idlocalitat) IS NOT NULL
      THEN
         BEGIN
            v_count := NULL;

            SELECT 1
              INTO v_count
              FROM localitat
             WHERE idlocalitat = p_idlocalitat AND ROWNUM = 1;
         EXCEPTION
            WHEN NO_DATA_FOUND
            THEN
               RAISE err_no_loc;
         END;
      END IF;

      -- Si compleix els requisits, se fa la inserci
      UPDATE COMPTADOR
         SET NIF = NVL (p_nif, NIF),
             POTENCIA = NVL (p_potencia, POTENCIA),
             DIRECCIO = NVL (p_direccio, DIRECCIO),
             IDLOCALITAT = NVL (p_idlocalitat, IDLOCALITAT),
             IDMODEL = NVL (p_idmodel, IDMODEL),
             DATA_LAST_MOD = SYSDATE
       WHERE IDCONTRACTE = p_idcontracte;

      RSP := 'OK: modificaci de comptador realitzada correctament.';

      INSERT INTO LOGS (NOMPROC,
                        LOGDATE,
                        PARIN,
                        PAROUT)
           VALUES ('pkg_abm.prc_modif_comptador',
                   SYSDATE,
                   v_parin,
                   'RSP = ' || RSP);

      COMMIT;
   EXCEPTION
      -- Si no se pot fer l'insert, se genera un registre de error a la taula LOGS
      WHEN err_null
      THEN
         RSP := 'ERROR: hi ha camps obligatoris sense informar';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_modif_comptador',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN err_no_idcontracte
      THEN
         RSP := 'ERROR: El comptador no existeix';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_modif_comptador',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN err_no_client
      THEN
         RSP := 'ERROR: El client no existeix a la base de dades';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_modif_comptador',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN err_no_loc
      THEN
         RSP := 'ERROR: La localitat no existeix a la base de dades';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_modif_comptador',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN OTHERS
      THEN
         err_num := SQLCODE;
         err_msg := SUBSTR (SQLERRM, 1, 200);

         RSP := 'Error no tractat: ' || TO_CHAR (err_num) || ': ' || err_msg;

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_modif_comptador',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
   END prc_modif_comptador;

   PROCEDURE prc_react_comptador (p_idcontracte IN comptador.idcontracte%TYPE, RSP OUT VARCHAR2)
   IS
      /*******************************************************************************************
          NOM:        prc_react_comptador
          PROPSIT:   Reactiva un comptador
          REQUISITS:  - S'ha d'informar l'Id del comptador
                      - El registre ha d'existir a la base de dades
          PARMETRES:
              IN:   Obligatoris:  p_idcontracte: Identificador del comptador
              OUT:  RSP
          PRE-CONDICI:  Existeix el comptador que se vol donar reactivar
          POST-CONDICI: se reactiva el comptador
          RETORNA:
              tot ha anat be:
                  RSP = 'OK: reactivaci de comptador realitzada correctament'
              missatge de errors:
                  'ERROR: hi ha camps obligatoris sense informar'
                  'ERROR: El comptador no existeix'
                  'ERROR: El registre ja est donat d'alta'
                  'ERROR: Error no tractat'
      *******************************************************************************************/
      err_null             EXCEPTION;
      err_no_idcontracte   EXCEPTION;
      err_compt_alta       EXCEPTION;

      v_count              VARCHAR2 (1);
      v_parin              VARCHAR2 (300);
      err_num              NUMBER;
      err_msg              VARCHAR2 (100);
   BEGIN
      v_parin := 'p_idcontracte: ' || p_idcontracte;

      -- Verificaci de requisits
      IF (TRIM (p_idcontracte) IS NULL)
      THEN
         RAISE err_null;
      END IF;

      -- Existeix Id a comptador
      BEGIN
         v_count := NULL;

         SELECT 1
           INTO v_count
           FROM comptador
          WHERE idcontracte = p_idcontracte AND ROWNUM = 1;
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            RAISE err_no_idcontracte;
      END;

      -- El registre no est donat de d'alta
      BEGIN
         v_count := NULL;

         SELECT 1
           INTO v_count
           FROM comptador
          WHERE idcontracte = p_idcontracte AND data_baixa IS NULL AND ROWNUM = 1;

         RAISE err_compt_alta;
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            NULL;
      END;

      -- Si compleix els requisits, s'elimina la data de baixa
      UPDATE COMPTADOR
         SET DATA_BAIXA = NULL
       WHERE idcontracte = p_idcontracte;

      RSP := 'OK: reactivaci de comptador realitzada correctament.';

      INSERT INTO LOGS (NOMPROC,
                        LOGDATE,
                        PARIN,
                        PAROUT)
           VALUES ('pkg_abm.prc_react_comptador',
                   SYSDATE,
                   v_parin,
                   'RSP = ' || RSP);

      COMMIT;
   EXCEPTION
      -- Si no se pot fer l'insert, se genera un registre de error a la taula LOGS
      WHEN err_null
      THEN
         RSP := 'ERROR: hi ha camps obligatoris sense informar';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_react_comptador',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN err_no_idcontracte
      THEN
         RSP := 'ERROR: El comptador no existeix';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_react_comptador',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN err_compt_alta
      THEN
         RSP := 'ERROR: El registre ja est donat de baixa';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_react_comptador',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN OTHERS
      THEN
         err_num := SQLCODE;
         err_msg := SUBSTR (SQLERRM, 1, 200);

         RSP := 'Error no tractat: ' || TO_CHAR (err_num) || ': ' || err_msg;

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_react_comptador',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
   END prc_react_comptador;

   /********************************** Linia_comunicacio **************************************/

   PROCEDURE prc_alta_linia_comunicacio (
      p_idcentralprod   IN     linia_comunicacio.idcentralprod%TYPE,
      p_capacitat_max   IN     linia_comunicacio.capacitat_max%TYPE,
      RSP                  OUT VARCHAR2)
   IS
      /*******************************************************************************************
          NOM:        prc_alta_linia_comunicacio
          PROPSIT:   Dona d'alta una lnia de comunicaci
          REQUISITS:  - Han d'estar informats tots els camps obligatoris (NOT NULL)
                      - El registre no ha d'existir
                      - L'Id de central de producci ha d'existir a la base de dades
          PARMETRES:
              IN:   Obligatoris: p_idcentralprod: Identificador de la central de producci
                                 p_capacitat_max: Capacitat mxima de la lnia
              OUT:  RSP
          PRE-CONDICI:  No existeix la lnia de comunicaci
          POST-CONDICI: Se dona d'alta la lnia de comunicaci
          RETORNA:
              tot ha anat be:
                  RSP = 'OK: alta de lnia realitzada correctament. Id = NumId'
              missatge de errors:
                  'ERROR: hi ha camps obligatoris sense informar'
                  'ERROR: La central de producci no existeix a la base de dades'
                  'ERROR: Error no tractat'
      *******************************************************************************************/
      err_null            EXCEPTION;
      err_no_idcentprod   EXCEPTION;

      v_count             VARCHAR2 (1);
      v_parin             VARCHAR2 (300);
      v_seq               NUMBER;
      err_num             NUMBER;
      err_msg             VARCHAR2 (100);
   BEGIN
      v_parin :=
         'p_idcentralprod: ' || p_idcentralprod || ', p_capacitat_max: ' || p_capacitat_max;

      -- Verificaci de requisits
      -- Han d'estar informats tots els camps obligatoris (NOT NULL)
      IF (TRIM (p_idcentralprod) IS NULL OR TRIM (p_capacitat_max) IS NULL)
      THEN
         RAISE err_null;
      END IF;

      -- Existeix Id a central_produccio
      BEGIN
         v_count := NULL;

         SELECT 1
           INTO v_count
           FROM central_produccio
          WHERE idcentralprod = p_idcentralprod AND ROWNUM = 1;
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            RAISE err_no_idcentprod;
      END;

      -- Si compleix els requisits, se fa la inserci
      INSERT INTO LINIA_COMUNICACIO (IDCENTRALPROD, CAPACITAT_MAX, DATA_ALTA)
           VALUES (p_idcentralprod, p_capacitat_max, SYSDATE);

      SELECT seq_LINIA_COMUNICACIO.CURRVAL INTO v_seq FROM DUAL;

      RSP := 'OK: alta de lnia de comunicaci realitzada correctament. Id = ' || v_seq;

      INSERT INTO LOGS (NOMPROC,
                        LOGDATE,
                        PARIN,
                        PAROUT)
           VALUES ('pkg_abm.prc_alta_linia_comunicacio',
                   SYSDATE,
                   v_parin,
                   'RSP = ' || RSP);

      COMMIT;
   EXCEPTION
      -- Si no se pot fer l'insert, se genera un registre de error a la taula LOGS
      WHEN err_null
      THEN
         RSP := 'ERROR: hi ha camps obligatoris sense informar';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_alta_linia_comunicacio',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN err_no_idcentprod
      THEN
         RSP := 'ERROR: La central de producci no existeix a la base de dades';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_alta_linia_comunicacio',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN OTHERS
      THEN
         err_num := SQLCODE;
         err_msg := SUBSTR (SQLERRM, 1, 200);

         RSP := 'Error no tractat: ' || TO_CHAR (err_num) || ': ' || err_msg;

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_alta_linia_comunicacio',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
   END prc_alta_linia_comunicacio;

   PROCEDURE prc_baixa_linia_comunicacio (p_idlinia   IN     linia_comunicacio.idlinia%TYPE,
                                          RSP            OUT VARCHAR2)
   IS
      /*******************************************************************************************
          NOM:        prc_baixa_linia_comunicacio
          PROPSIT:   Dona de baixa una lnia de comunicaci
          REQUISITS:  - S'ha d'informar l'Id de la lnia de comunicaci
                      - El registre ha d'existir a la base de dades
                      - El registre no ha d'estar donat de baixa
          PARMETRES:
              IN:   Obligatoris:  p_idlinia: Identificador de la lnia de comunicaci
              OUT:  RSP
          PRE-CONDICI:  Existeix la lnia de comunicaci que se vol donar de baixa
          POST-CONDICI: se dona de baixa la lnia de comunicaci
          RETORNA:
              tot ha anat be:
                  RSP = 'OK: baixa de lnia de comunicaci realitzada correctament'
              missatge de errors:
                  'ERROR: hi ha camps obligatoris sense informar'
                  'ERROR: La lnia de comunicaci no existeix'
                  'ERROR: El registre ja est donat de baixa'
                  'ERROR: Error no tractat'
      *******************************************************************************************/
      err_null          EXCEPTION;
      err_no_idlinia    EXCEPTION;
      err_linia_baixa   EXCEPTION;

      v_count           VARCHAR2 (1);
      v_parin           VARCHAR2 (300);
      err_num           NUMBER;
      err_msg           VARCHAR2 (100);
   BEGIN
      v_parin := 'p_idlinia: ' || p_idlinia;

      -- Verificaci de requisits
      IF (TRIM (p_idlinia) IS NULL)
      THEN
         RAISE err_null;
      END IF;

      -- Existeix Id a linia_comunicacio
      BEGIN
         v_count := NULL;

         SELECT 1
           INTO v_count
           FROM linia_comunicacio
          WHERE idlinia = p_idlinia AND ROWNUM = 1;
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            RAISE err_no_idlinia;
      END;

      -- El registre no est donat de baixa
      BEGIN
         v_count := NULL;

         SELECT 1
           INTO v_count
           FROM linia_comunicacio
          WHERE idlinia = p_idlinia AND data_baixa IS NOT NULL AND ROWNUM = 1;

         RAISE err_linia_baixa;
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            NULL;
      END;

      -- Si compleix els requisits, s'afegeix la data de baixa
      UPDATE LINIA_COMUNICACIO
         SET DATA_BAIXA = SYSDATE
       WHERE idlinia = p_idlinia;

      RSP := 'OK: baixa de lnia de comunicaci realitzada correctament.';

      INSERT INTO LOGS (NOMPROC,
                        LOGDATE,
                        PARIN,
                        PAROUT)
           VALUES ('pkg_abm.prc_baixa_linia_comunicacio',
                   SYSDATE,
                   v_parin,
                   'RSP = ' || RSP);

      COMMIT;
   EXCEPTION
      -- Si no se pot fer l'insert, se genera un registre de error a la taula LOGS
      WHEN err_null
      THEN
         RSP := 'ERROR: hi ha camps obligatoris sense informar';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_baixa_linia_comunicacio',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN err_no_idlinia
      THEN
         RSP := 'ERROR: La lnia de comunicaci no existeix';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_baixa_linia_comunicacio',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN err_linia_baixa
      THEN
         RSP := 'ERROR: El registre ja est donat de baixa';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_baixa_linia_comunicacio',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN OTHERS
      THEN
         err_num := SQLCODE;
         err_msg := SUBSTR (SQLERRM, 1, 200);

         RSP := 'Error no tractat: ' || TO_CHAR (err_num) || ': ' || err_msg;

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_baixa_linia_comunicacio',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
   END prc_baixa_linia_comunicacio;

   PROCEDURE prc_modif_linia_comunicacio (
      p_idlinia         IN     linia_comunicacio.idlinia%TYPE,
      p_idcentralprod   IN     linia_comunicacio.idcentralprod%TYPE,
      p_capacitat_max   IN     linia_comunicacio.capacitat_max%TYPE,
      RSP                  OUT VARCHAR2)
   IS
      /*******************************************************************************************
          NOM:        prc_modif_linia_comunicacio
          PROPSIT:   Modifica una lnia de comunicaci
          REQUISITS:  - S'ha d'informar l'Id de la lnia de comunicaci
                      - Existeix la central de producci que se vol modificar
          PARMETRES:
              IN:   Obligatoris:  p_idlinia: Identificador de la lnia de comunicaci
                    Opcionals:    p_idcentralprod: Identificador de la central de producci
                                  p_capacitat_max: Capacitat mxima de la lnia
              OUT:  RSP
          PRE-CONDICI:  Existeix la lnia de comunicaci que se vol modificar
          POST-CONDICI: Se modifica la lnia de comunicaci
          RETORNA:
              tot ha anat be:
                  RSP = 'OK: modificaci de lnia de comunicaci realitzada correctament'
              missatge de errors:
                  'ERROR: hi ha camps obligatoris sense informar'
                  'ERROR: la lnia de comunicaci no existeix'
                  'ERROR: Error no tractat'
      *******************************************************************************************/
      err_null            EXCEPTION;
      err_no_idlinia      EXCEPTION;
      err_no_idcentprod   EXCEPTION;

      v_count             VARCHAR2 (1);
      v_parin             VARCHAR2 (300);
      err_num             NUMBER;
      err_msg             VARCHAR2 (100);
   BEGIN
      v_parin :=
            'p_idcentralprod: '
         || p_idcentralprod
         || ', p_idlinia '
         || p_idlinia
         || ', p_capacitat_max: '
         || p_capacitat_max;

      -- Verificaci de requisits
      -- Han d'estar informats tots els camps obligatoris (NOT NULL)
      IF (TRIM (p_idlinia) IS NULL)
      THEN
         RAISE err_null;
      END IF;

      -- Existeix Id a linia_comunicacio
      BEGIN
         v_count := NULL;

         SELECT 1
           INTO v_count
           FROM linia_comunicacio
          WHERE idlinia = p_idlinia AND ROWNUM = 1;
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            RAISE err_no_idlinia;
      END;

      -- Existeix Id a central_produccio
      IF p_idcentralprod IS NOT NULL
      THEN
         BEGIN
            v_count := NULL;

            SELECT 1
              INTO v_count
              FROM central_produccio
             WHERE idcentralprod = p_idcentralprod AND ROWNUM = 1;
         EXCEPTION
            WHEN NO_DATA_FOUND
            THEN
               RAISE err_no_idcentprod;
         END;
      END IF;

      -- Si compleix els requisits, se fa la inserci
      UPDATE LINIA_COMUNICACIO
         SET IDCENTRALPROD = NVL (p_idcentralprod, IDCENTRALPROD),
             CAPACITAT_MAX = NVL (p_capacitat_max, CAPACITAT_MAX),
             DATA_LAST_MOD = SYSDATE
       WHERE IDLINIA = p_idlinia;

      RSP := 'OK: modificaci de lnia de comunicaci realitzada correctament.';

      INSERT INTO LOGS (NOMPROC,
                        LOGDATE,
                        PARIN,
                        PAROUT)
           VALUES ('pkg_abm.prc_modif_linia_comunicacio',
                   SYSDATE,
                   v_parin,
                   'RSP = ' || RSP);

      COMMIT;
   EXCEPTION
      -- Si no se pot fer l'insert, se genera un registre de error a la taula LOGS
      WHEN err_null
      THEN
         RSP := 'ERROR: hi ha camps obligatoris sense informar';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_modif_linia_comunicacio',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN err_no_idlinia
      THEN
         RSP := 'ERROR: la lnia de comunicaci no existeix';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_modif_linia_comunicacio',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN err_no_idcentprod
      THEN
         RSP := 'ERROR: La central de producci no existeix a la base de dades';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_modif_linia_comunicacio',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN OTHERS
      THEN
         err_num := SQLCODE;
         err_msg := SUBSTR (SQLERRM, 1, 200);

         RSP := 'Error no tractat: ' || TO_CHAR (err_num) || ': ' || err_msg;

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_modif_central_distribucio',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
   END prc_modif_linia_comunicacio;

   PROCEDURE prc_react_linia_comunicacio (p_idlinia   IN     linia_comunicacio.idlinia%TYPE,
                                          RSP            OUT VARCHAR2)
   IS
      /*******************************************************************************************
          NOM:        prc_react_linia_comunicacio
          PROPSIT:   Reactiva una lnia de comunicaci
          REQUISITS:  - S'ha d'informar l'Id de la lnia de comunicaci
                      - El registre ha d'existir a la base de dades
          PARMETRES:
              IN:   Obligatoris:  p_idlinia: Identificador de la lnia de comunicaci
              OUT:  RSP
          PRE-CONDICI:  Existeix la lnia de comunicaci que se vol reactivar
          POST-CONDICI: se reactiva la lnia de comunicaci
          RETORNA:
              tot ha anat be:
                  RSP = 'OK: reactivaci de lnia de comunicaci realitzada correctament'
              missatge de errors:
                  'ERROR: hi ha camps obligatoris sense informar'
                  'ERROR: La lnia de comunicaci no existeix'
                  'ERROR: El registre ja est donat d'alta'
                  'ERROR: Error no tractat'
      *******************************************************************************************/
      err_null         EXCEPTION;
      err_no_idlinia   EXCEPTION;
      err_linia_alta   EXCEPTION;

      v_count          VARCHAR2 (1);
      v_parin          VARCHAR2 (300);
      err_num          NUMBER;
      err_msg          VARCHAR2 (100);
   BEGIN
      v_parin := 'p_idlinia: ' || p_idlinia;

      -- Verificaci de requisits
      IF (TRIM (p_idlinia) IS NULL)
      THEN
         RAISE err_null;
      END IF;

      -- Existeix Id a linia_comunicacio
      BEGIN
         v_count := NULL;

         SELECT 1
           INTO v_count
           FROM linia_comunicacio
          WHERE idlinia = p_idlinia AND ROWNUM = 1;
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            RAISE err_no_idlinia;
      END;

      -- El registre no est donat d'alta
      BEGIN
         v_count := NULL;

         SELECT 1
           INTO v_count
           FROM linia_comunicacio
          WHERE idlinia = p_idlinia AND data_baixa IS NULL AND ROWNUM = 1;

         RAISE err_linia_alta;
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            NULL;
      END;

      -- Si compleix els requisits, s'elimina la data de baixa
      UPDATE LINIA_COMUNICACIO
         SET DATA_BAIXA = NULL
       WHERE idlinia = p_idlinia;

      RSP := 'OK: reactivaci de lnia de comunicaci realitzada correctament.';

      INSERT INTO LOGS (NOMPROC,
                        LOGDATE,
                        PARIN,
                        PAROUT)
           VALUES ('pkg_abm.prc_react_linia_comunicacio',
                   SYSDATE,
                   v_parin,
                   'RSP = ' || RSP);

      COMMIT;
   EXCEPTION
      -- Si no se pot fer l'insert, se genera un registre de error a la taula LOGS
      WHEN err_null
      THEN
         RSP := 'ERROR: hi ha camps obligatoris sense informar';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_react_linia_comunicacio',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN err_no_idlinia
      THEN
         RSP := 'ERROR: La lnia de comunicaci no existeix';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_react_linia_comunicacio',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN err_linia_alta
      THEN
         RSP := 'ERROR: El registre ja est donat de baixa';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_react_linia_comunicacio',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN OTHERS
      THEN
         err_num := SQLCODE;
         err_msg := SUBSTR (SQLERRM, 1, 200);

         RSP := 'Error no tractat: ' || TO_CHAR (err_num) || ': ' || err_msg;

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_react_linia_comunicacio',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
   END prc_react_linia_comunicacio;

   /*************************************** Client ********************************************/

   PROCEDURE prc_alta_client (p_nif              IN     client.nif%TYPE,
                              p_tipusclient      IN     client.tipus_client%TYPE,
                              p_nom              IN     client.nom%TYPE,
                              p_cognom1          IN     client.cognom1%TYPE,
                              p_cognom2          IN     client.cognom2%TYPE,
                              p_data_naixement   IN     client.data_naixement%TYPE,
                              p_direccio         IN     client.direccio%TYPE,
                              p_idlocalitat      IN     client.idlocalitat%TYPE,
                              RSP                   OUT VARCHAR2)
   IS
      /*******************************************************************************************
          NOM:        prc_alta_client
          PROPSIT:   Dona d'alta un client
          REQUISITS:  - Han d'estar informats tots els camps obligatoris (NOT NULL)
                      - El registre no ha d'existir
                      - L'Id de localitat ha d'existir a la base de dades
          PARMETRES:
              IN:   Obligatoris:  p_nif: NIF(Identificador) del client
                                  p_tipusclient: indica si el client es una persona (1) o una empresa (2)
                                  p_nom: Nom del client
                                  p_direccio: Adrea del client
                                  p_idlocalitat: Identificador de la localitat de la direcci
                    Opcionals:    p_data_naixement: Data de naixement de el client
                                  p_cognom1: Primer cognom del client
                                  p_cognom2: Segon cognom del client
              OUT:  RSP
          PRE-CONDICI:  No existeix el client
          POST-CONDICI: Se dona d'alta el client
          RETORNA:
              tot ha anat be:
                  RSP = 'OK: alta de client realitzada correctament. Id = NIF'
              missatge de errors:
                  'ERROR: hi ha camps obligatoris sense informar'
                  'ERROR: El client ja existeix'
                  'ERROR: La localitat no existeix a la base de dades'
                  'ERROR: Error no tractat'
      *******************************************************************************************/
      err_null     EXCEPTION;
      err_nif      EXCEPTION;
      err_no_loc   EXCEPTION;

      v_count      VARCHAR2 (1);
      v_parin      VARCHAR2 (300);
      err_num      NUMBER;
      err_msg      VARCHAR2 (100);
   BEGIN
      v_parin :=
            'p_nif: '
         || p_nif
         || ', p_tipusclient: '
         || p_tipusclient
         || ', p_nom: '
         || p_nom
         || ', p_cognom1: '
         || p_cognom1
         || ', p_cognom2: '
         || p_cognom2
         || ', p_direccio: '
         || p_direccio
         || ', p_idlocalitat: '
         || p_idlocalitat
         || ', p_data_naixement: '
         || p_data_naixement;

      -- Verificaci de requisits
      -- Han d'estar informats tots els camps obligatoris (NOT NULL)
      IF (   TRIM (p_nif) IS NULL
          OR TRIM (p_tipusclient) IS NULL
          OR TRIM (p_nom) IS NULL
          OR TRIM (p_direccio) IS NULL
          OR TRIM (p_idlocalitat) IS NULL)
      THEN
         RAISE err_null;
      END IF;

      -- Existeix NIF del client
      BEGIN
         v_count := NULL;

         SELECT 1
           INTO v_count
           FROM client
          WHERE nif = p_nif AND ROWNUM = 1;

         RAISE err_nif;
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            NULL;
      END;

      -- Existeix Id a localitat
      BEGIN
         v_count := NULL;

         SELECT 1
           INTO v_count
           FROM localitat
          WHERE idlocalitat = p_idlocalitat AND ROWNUM = 1;
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            RAISE err_no_loc;
      END;

      -- Si compleix els requisits, se fa la inserci
      INSERT INTO CLIENT (NIF,
                          TIPUS_CLIENT,
                          NOM,
                          COGNOM1,
                          COGNOM2,
                          DIRECCIO,
                          IDLOCALITAT,
                          DATA_NAIXEMENT,
                          DATA_ALTA)
           VALUES (p_nif,
                   p_tipusclient,
                   p_nom,
                   p_cognom1,
                   p_cognom2,
                   p_direccio,
                   p_idlocalitat,
                   p_data_naixement,
                   SYSDATE);

      RSP := 'OK: alta de client realitzada correctament. Id = ' || p_nif;

      INSERT INTO LOGS (NOMPROC,
                        LOGDATE,
                        PARIN,
                        PAROUT)
           VALUES ('pkg_abm.prc_alta_client',
                   SYSDATE,
                   v_parin,
                   'RSP = ' || RSP);

      COMMIT;
   EXCEPTION
      -- Si no se pot fer l'insert, se genera un registre de error a la taula LOGS
      WHEN err_null
      THEN
         RSP := 'ERROR: hi ha camps obligatoris sense informar';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_alta_client',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN err_nif
      THEN
         RSP := 'ERROR: El client ja existeix';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_alta_client',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN err_no_loc
      THEN
         RSP := 'ERROR: La localitat no existeix a la base de dades';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_alta_client',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN OTHERS
      THEN
         err_num := SQLCODE;
         err_msg := SUBSTR (SQLERRM, 1, 200);

         RSP := 'Error no tractat: ' || TO_CHAR (err_num) || ': ' || err_msg;

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_alta_client',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
   END prc_alta_client;

   PROCEDURE prc_baixa_client (p_nif IN client.nif%TYPE, RSP OUT VARCHAR2)
   IS
      /*******************************************************************************************
          NOM:        prc_baixa_client
          PROPSIT:   Dona de baixa un client
          REQUISITS:  - S'ha d'informar el NIF del client
                      - El registre ha d'existir a la base de dades
                      - El registre no ha d'estar donat de baixa
          PARMETRES:
              IN:   Obligatoris:  p_nif: NIF(Identificador) del client
              OUT:  RSP
          PRE-CONDICI:  Existeix el client que se vol donar de baixa
          POST-CONDICI: se dona de baixa el client
          RETORNA:
              tot ha anat be:
                  RSP = 'OK: baixa de client realitzada correctament'
              missatge de errors:
                  'ERROR: hi ha camps obligatoris sense informar'
                  'ERROR: El client no existeix'
                  'ERROR: El registre ja est donat de baixa'
                  'ERROR: Error no tractat'
      *******************************************************************************************/
      err_null           EXCEPTION;
      err_no_nif         EXCEPTION;
      err_client_baixa   EXCEPTION;

      v_count            VARCHAR2 (1);
      v_parin            VARCHAR2 (300);
      err_num            NUMBER;
      err_msg            VARCHAR2 (100);
   BEGIN
      v_parin := 'p_nif: ' || p_nif;

      -- Verificaci de requisits
      IF (TRIM (p_nif) IS NULL)
      THEN
         RAISE err_null;
      END IF;

      -- Existeix Id a client
      BEGIN
         v_count := NULL;

         SELECT 1
           INTO v_count
           FROM client
          WHERE nif = p_nif AND ROWNUM = 1;
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            RAISE err_no_nif;
      END;

      -- El registre no est donat de baixa
      BEGIN
         v_count := NULL;

         SELECT 1
           INTO v_count
           FROM client
          WHERE nif = p_nif AND data_baixa IS NOT NULL AND ROWNUM = 1;

         RAISE err_client_baixa;
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            NULL;
      END;

      -- Si compleix els requisits, s'afegeix la data de baixa
      UPDATE CLIENT
         SET DATA_BAIXA = SYSDATE
       WHERE nif = p_nif;

      RSP := 'OK: baixa de client realitzada correctament.';

      INSERT INTO LOGS (NOMPROC,
                        LOGDATE,
                        PARIN,
                        PAROUT)
           VALUES ('pkg_abm.prc_baixa_client',
                   SYSDATE,
                   v_parin,
                   'RSP = ' || RSP);

      COMMIT;
   EXCEPTION
      -- Si no se pot fer l'insert, se genera un registre de error a la taula LOGS
      WHEN err_null
      THEN
         RSP := 'ERROR: hi ha camps obligatoris sense informar';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_baixa_client',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN err_no_nif
      THEN
         RSP := 'ERROR: El client no existeix';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_baixa_client',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN err_client_baixa
      THEN
         RSP := 'ERROR: El registre ja est donat de baixa';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_baixa_client',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN OTHERS
      THEN
         err_num := SQLCODE;
         err_msg := SUBSTR (SQLERRM, 1, 200);

         RSP := 'Error no tractat: ' || TO_CHAR (err_num) || ': ' || err_msg;

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_baixa_client',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
   END prc_baixa_client;

   PROCEDURE prc_modif_client (p_nif              IN     client.nif%TYPE,
                               p_tipusclient      IN     client.tipus_client%TYPE,
                               p_nom              IN     client.nom%TYPE,
                               p_cognom1          IN     client.cognom1%TYPE,
                               p_cognom2          IN     client.cognom2%TYPE,
                               p_data_naixement   IN     client.data_naixement%TYPE,
                               p_direccio         IN     client.direccio%TYPE,
                               p_idlocalitat      IN     client.idlocalitat%TYPE,
                               RSP                   OUT VARCHAR2)
   IS
      /*******************************************************************************************
          NOM:        prc_modif_client
          PROPSIT:   Modifica un client
          REQUISITS:  - S'ha d'informar l'Id del client
                      - L'Id de localitat ha d'existir a la base de dades
          PARMETRES:
              IN:   Obligatoris:  p_nif: NIF(Identificador) del client
                    Opcionals:    p_tipusclient: indica si el client es una persona (1) o una empresa (2)
                                  p_nom: Nom del client
                                  p_cognom1: Primer cognom del client
                                  p_cognom2: Segon cognom del client
                                  p_direccio: Adrea del client
                                  p_idlocalitat: Identificador de la localitat de la direcci
                                  p_data_naixement: Data de naixement de el client
              OUT:  RSP
          PRE-CONDICI:  Existeix el client que se vol modificar
          POST-CONDICI: Se modifica el client
          RETORNA:
              tot ha anat be:
                  RSP = 'OK: modificaci de client realitzada correctament'
              missatge de errors:
                  'ERROR: hi ha camps obligatoris sense informar'
                  'ERROR: El client no existeix'
                  'ERROR: La localitat no existeix a la base de dades'
                  'ERROR: Error no tractat'
      *******************************************************************************************/
      err_null     EXCEPTION;
      err_no_nif   EXCEPTION;
      err_no_loc   EXCEPTION;

      v_count      VARCHAR2 (1);
      v_parin      VARCHAR2 (300);
      err_num      NUMBER;
      err_msg      VARCHAR2 (100);
   BEGIN
      v_parin :=
            'p_nif: '
         || p_nif
         || ', p_tipusclient: '
         || p_tipusclient
         || ', p_nom: '
         || p_nom
         || ', p_cognom1: '
         || p_cognom1
         || ', p_cognom2: '
         || p_cognom2
         || ', p_direccio: '
         || p_direccio
         || ', p_idlocalitat: '
         || p_idlocalitat
         || ', p_data_naixement: '
         || p_data_naixement;

      -- Verificaci de requisits
      -- Han d'estar informats tots els camps obligatoris (NOT NULL)
      IF (TRIM (p_nif) IS NULL)
      THEN
         RAISE err_null;
      END IF;

      -- Existeix NIF del client
      BEGIN
         v_count := NULL;

         SELECT 1
           INTO v_count
           FROM client
          WHERE nif = p_nif AND ROWNUM = 1;
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            RAISE err_no_nif;
      END;

      -- Existeix Id a localitat
      IF TRIM (p_idlocalitat) IS NOT NULL
      THEN
         BEGIN
            v_count := NULL;

            SELECT 1
              INTO v_count
              FROM localitat
             WHERE idlocalitat = p_idlocalitat AND ROWNUM = 1;
         EXCEPTION
            WHEN NO_DATA_FOUND
            THEN
               RAISE err_no_loc;
         END;
      END IF;

      -- Si compleix els requisits, se fa la inserci
      UPDATE CLIENT
         SET TIPUS_CLIENT = NVL (p_tipusclient, TIPUS_CLIENT),
             NOM = NVL (p_nom, NOM),
             COGNOM1 = NVL (p_cognom1, COGNOM1),
             COGNOM2 = NVL (p_cognom2, COGNOM2),
             DIRECCIO = NVL (p_direccio, DIRECCIO),
             IDLOCALITAT = NVL (p_idlocalitat, IDLOCALITAT),
             DATA_NAIXEMENT = NVL (p_data_naixement, DATA_NAIXEMENT),
             DATA_LAST_MOD = SYSDATE
       WHERE NIF = p_nif;

      RSP := 'OK: modificaci de client realitzada correctament.';

      INSERT INTO LOGS (NOMPROC,
                        LOGDATE,
                        PARIN,
                        PAROUT)
           VALUES ('pkg_abm.prc_modif_client',
                   SYSDATE,
                   v_parin,
                   'RSP = ' || RSP);

      COMMIT;
   EXCEPTION
      -- Si no se pot fer l'insert, se genera un registre de error a la taula LOGS
      WHEN err_null
      THEN
         RSP := 'ERROR: hi ha camps obligatoris sense informar';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_modif_client',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN err_no_nif
      THEN
         RSP := 'ERROR: El client no existeix';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_modif_client',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN err_no_loc
      THEN
         RSP := 'ERROR: La localitat no existeix a la base de dades';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_modif_client',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN OTHERS
      THEN
         err_num := SQLCODE;
         err_msg := SUBSTR (SQLERRM, 1, 200);

         RSP := 'Error no tractat: ' || TO_CHAR (err_num) || ': ' || err_msg;

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_modif_client',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
   END prc_modif_client;

   PROCEDURE prc_react_client (p_nif IN client.nif%TYPE, RSP OUT VARCHAR2)
   IS
      /*******************************************************************************************
          NOM:        prc_react_client
          PROPSIT:   Reactiva un client
          REQUISITS:  - S'ha d'informar el NIF del client
                      - El registre ha d'existir a la base de dades
          PARMETRES:
              IN:   Obligatoris:  p_nif: NIF(Identificador) del client
              OUT:  RSP
          PRE-CONDICI:  Existeix el client que se vol reactivar
          POST-CONDICI: se reactiva el client
          RETORNA:
              tot ha anat be:
                  RSP = 'OK: reactivaci de client realitzada correctament'
              missatge de errors:
                  'ERROR: hi ha camps obligatoris sense informar'
                  'ERROR: El client no existeix'
                  'ERROR: El registre ja est donat d'alta'
                  'ERROR: Error no tractat'
      *******************************************************************************************/
      err_null          EXCEPTION;
      err_no_nif        EXCEPTION;
      err_client_alta   EXCEPTION;

      v_count           VARCHAR2 (1);
      v_parin           VARCHAR2 (300);
      err_num           NUMBER;
      err_msg           VARCHAR2 (100);
   BEGIN
      v_parin := 'p_nif: ' || p_nif;

      -- Verificaci de requisits
      IF (TRIM (p_nif) IS NULL)
      THEN
         RAISE err_null;
      END IF;

      -- Existeix Id a client
      BEGIN
         v_count := NULL;

         SELECT 1
           INTO v_count
           FROM client
          WHERE nif = p_nif AND ROWNUM = 1;
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            RAISE err_no_nif;
      END;

      -- El registre no est donat d'alta
      BEGIN
         v_count := NULL;

         SELECT 1
           INTO v_count
           FROM client
          WHERE nif = p_nif AND data_baixa IS NULL AND ROWNUM = 1;

         RAISE err_client_alta;
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            NULL;
      END;

      -- Si compleix els requisits, s'afegeix la data de baixa
      UPDATE CLIENT
         SET DATA_BAIXA = NULL
       WHERE nif = p_nif;

      RSP := 'OK: reactivaci de client realitzada correctament.';

      INSERT INTO LOGS (NOMPROC,
                        LOGDATE,
                        PARIN,
                        PAROUT)
           VALUES ('pkg_abm.prc_react_client',
                   SYSDATE,
                   v_parin,
                   'RSP = ' || RSP);

      COMMIT;
   EXCEPTION
      -- Si no se pot fer l'insert, se genera un registre de error a la taula LOGS
      WHEN err_null
      THEN
         RSP := 'ERROR: hi ha camps obligatoris sense informar';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_react_client',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN err_no_nif
      THEN
         RSP := 'ERROR: El client no existeix';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_react_client',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN err_client_alta
      THEN
         RSP := 'ERROR: El registre ja est donat de baixa';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_react_client',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN OTHERS
      THEN
         err_num := SQLCODE;
         err_msg := SUBSTR (SQLERRM, 1, 200);

         RSP := 'Error no tractat: ' || TO_CHAR (err_num) || ': ' || err_msg;

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_react_client',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
   END prc_react_client;

   /************************************** Poblacio *******************************************/

   PROCEDURE prc_alta_poblacio (p_nompoblacio   IN     poblacio.nompoblacio%TYPE,
                                p_idpais        IN     poblacio.idpais%TYPE,
                                RSP                OUT VARCHAR2)
   IS
      /*******************************************************************************************
          NOM:        prc_alta_poblacio
          PROPSIT:   Dona d'alta una poblaci
          REQUISITS:  - Han d'estar informats tots els camps obligatoris (NOT NULL)
                      - El registre no ha d'existir
                      - L'Id de pas ha d'existir a la base de dades
          PARMETRES:
              IN:   Obligatoris:  p_nompoblacio: Nom de la poblaci
                                  p_idpais: Identificador del pas
              OUT:  RSP
          PRE-CONDICI:  No existeix la poblaci
          POST-CONDICI: Se dona d'alta la poblaci
          RETORNA:
              tot ha anat be:
                  RSP = 'OK: alta de poblaci realitzada correctament. Id = NumId'
              missatge de errors:
                  'ERROR: hi ha camps obligatoris sense informar'
                  'ERROR: La poblaci ja existeix'
                  'ERROR: El pas no existeix a la base de dades'
                  'ERROR: Error no tractat'
      *******************************************************************************************/
      err_null          EXCEPTION;
      err_nompoblacio   EXCEPTION;
      err_no_pais       EXCEPTION;

      v_count           VARCHAR2 (1);
      v_parin           VARCHAR2 (300);
      v_seq             NUMBER;
      err_num           NUMBER;
      err_msg           VARCHAR2 (100);
   BEGIN
      v_parin := 'p_nompoblacio: ' || p_nompoblacio || ', p_idpais: ' || p_idpais;

      -- Verificaci de requisits
      -- Han d'estar informats tots els camps obligatoris (NOT NULL)
      IF (TRIM (p_idpais) IS NULL OR TRIM (p_idpais) IS NULL)
      THEN
         RAISE err_null;
      END IF;

      -- Existeix la poblacio
      BEGIN
         v_count := NULL;

         SELECT 1
           INTO v_count
           FROM poblacio
          WHERE nompoblacio = p_nompoblacio AND ROWNUM = 1;

         RAISE err_nompoblacio;
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            NULL;
      END;

      -- Existeix Id a pais
      BEGIN
         v_count := NULL;

         SELECT 1
           INTO v_count
           FROM pais
          WHERE idpais = p_idpais AND ROWNUM = 1;
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            RAISE err_no_pais;
      END;

      -- Si compleix els requisits, se fa la inserci
      INSERT INTO POBLACIO (NOMPOBLACIO, IDPAIS)
           VALUES (p_nompoblacio, p_idpais);

      SELECT seq_POBLACIO.CURRVAL INTO v_seq FROM DUAL;

      RSP := 'OK: alta de poblaci realitzada correctament. Id = ' || v_seq;

      INSERT INTO LOGS (NOMPROC,
                        LOGDATE,
                        PARIN,
                        PAROUT)
           VALUES ('pkg_abm.prc_alta_poblacio',
                   SYSDATE,
                   v_parin,
                   'RSP = ' || RSP);

      COMMIT;
   EXCEPTION
      -- Si no se pot fer l'insert, se genera un registre de error a la taula LOGS
      WHEN err_null
      THEN
         RSP := 'ERROR: hi ha camps obligatoris sense informar';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_alta_poblacio',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN err_nompoblacio
      THEN
         RSP := 'ERROR: La poblaci ja existeix';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_alta_poblacio',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN err_no_pais
      THEN
         RSP := 'ERROR: El pas no existeix a la base de dades';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_alta_poblacio',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN OTHERS
      THEN
         err_num := SQLCODE;
         err_msg := SUBSTR (SQLERRM, 1, 200);

         RSP := 'Error no tractat: ' || TO_CHAR (err_num) || ': ' || err_msg;

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_alta_poblacio',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
   END prc_alta_poblacio;

   PROCEDURE prc_borra_poblacio (p_idpoblacio IN poblacio.idpoblacio%TYPE, RSP OUT VARCHAR2)
   IS
      /*******************************************************************************************
          NOM:        prc_borra_poblacio
          PROPSIT:   Esborra una poblaci
          REQUISITS:  - S'ha d'informar l'Id de la poblaci
                      - El registre ha d'existir a la base de dades
                      - El registre no ha d'estar lligat a cap taula externa (localitat)
          PARMETRES:
              IN:   Obligatoris:  p_idpoblacio: Identificador de la poblaci
              OUT:  RSP
          PRE-CONDICI:  Existeix la poblaci que se vol borrar
          POST-CONDICI: se borra la poblaci
          RETORNA:
              tot ha anat be:
                  RSP = 'OK: Delete de poblaci realitzat correctament'
              missatge de errors:
                  'ERROR: hi ha camps obligatoris sense informar'
                  'ERROR: La poblaci no existeix'
                  'ERROR: Hi ha localitats lligades a aquesta poblaci'
                  'ERROR: Error no tractat'
      *******************************************************************************************/
      err_null              EXCEPTION;
      err_no_idpoblacio     EXCEPTION;
      err_idpob_localitat   EXCEPTION;

      v_count               VARCHAR2 (1);
      v_parin               VARCHAR2 (300);
      err_num               NUMBER;
      err_msg               VARCHAR2 (100);
   BEGIN
      v_parin := 'p_idpoblacio: ' || p_idpoblacio;

      -- Verificaci de requisits
      IF (TRIM (p_idpoblacio) IS NULL)
      THEN
         RAISE err_null;
      END IF;

      -- Existeix Id a poblacio
      BEGIN
         v_count := NULL;

         SELECT 1
           INTO v_count
           FROM poblacio
          WHERE idpoblacio = p_idpoblacio AND ROWNUM = 1;
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            RAISE err_no_idpoblacio;
      END;

      -- El registre no est lligat a localitat
      BEGIN
         v_count := NULL;

         SELECT 1
           INTO v_count
           FROM localitat
          WHERE idpoblacio = p_idpoblacio AND ROWNUM = 1;

         RAISE err_idpob_localitat;
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            NULL;
      END;

      -- Si compleix els requisits, se fa un delete de la poblacio
      DELETE FROM POBLACIO
            WHERE IDPOBLACIO = p_idpoblacio;

      RSP := 'OK: delete de poblacio realitzat correctament.';

      INSERT INTO LOGS (NOMPROC,
                        LOGDATE,
                        PARIN,
                        PAROUT)
           VALUES ('pkg_abm.prc_borra_poblacio',
                   SYSDATE,
                   v_parin,
                   'RSP = ' || RSP);

      COMMIT;
   EXCEPTION
      -- Si no se pot fer l'insert, se genera un registre de error a la taula LOGS
      WHEN err_null
      THEN
         RSP := 'ERROR: hi ha camps obligatoris sense informar';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_borra_poblacio',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN err_no_idpoblacio
      THEN
         RSP := 'ERROR: La poblaci no existeix';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_borra_poblacio',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN err_idpob_localitat
      THEN
         RSP := 'ERROR: Hi ha localitats lligades a aquesta poblaci';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_borra_poblacio',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN OTHERS
      THEN
         err_num := SQLCODE;
         err_msg := SUBSTR (SQLERRM, 1, 200);

         RSP := 'Error no tractat: ' || TO_CHAR (err_num) || ': ' || err_msg;

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_borra_poblacio',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
   END prc_borra_poblacio;

   PROCEDURE prc_modif_poblacio (p_idpoblacio    IN     poblacio.idpoblacio%TYPE,
                                 p_nompoblacio   IN     poblacio.nompoblacio%TYPE,
                                 p_idpais        IN     poblacio.idpais%TYPE,
                                 RSP                OUT VARCHAR2)
   IS
      /*******************************************************************************************
          NOM:        prc_modif_poblacio
          PROPSIT:   Modifica una poblaci
          REQUISITS:  - S'ha d'informar l'Id de la poblaci
                      - L'Id del pas ha d'existir a la base de dades
          PARMETRES:
              IN:   Obligatoris:  p_idpoblacio: Identificador de la poblaci
                    Opcionals:    p_nompoblacio: Nom de la poblaci
                                  p_idpais: Identificador del pas
              OUT:  RSP
          PRE-CONDICI:  Existeix la poblaci que se vol modificar
          POST-CONDICI: Se modifica la poblaci
          RETORNA:
              tot ha anat be:
                  RSP = 'OK: modificaci de poblaci realitzada correctament'
              missatge de errors:
                  'ERROR: hi ha camps obligatoris sense informar'
                  'ERROR: La poblaci no existeix'
                  'ERROR: El nom de la poblaci ja existeix'
                  'ERROR: El pas no existeix a la base de dades'
                  'ERROR: Error no tractat'
      *******************************************************************************************/
      err_null          EXCEPTION;
      err_no_idpoblac   EXCEPTION;
      err_nompoblacio   EXCEPTION;
      err_no_pais       EXCEPTION;

      v_count           VARCHAR2 (1);
      v_parin           VARCHAR2 (300);
      err_num           NUMBER;
      err_msg           VARCHAR2 (100);
   BEGIN
      v_parin :=
            'p_idpoblacio: '
         || p_idpoblacio
         || 'p_nompoblacio: '
         || p_nompoblacio
         || ', p_idpais: '
         || p_idpais;

      -- Verificaci de requisits
      -- Han d'estar informats tots els camps obligatoris (NOT NULL)
      IF (TRIM (p_idpoblacio) IS NULL)
      THEN
         RAISE err_null;
      END IF;

      -- Existeix l'Id poblacio
      BEGIN
         v_count := NULL;

         SELECT 1
           INTO v_count
           FROM poblacio
          WHERE idpoblacio = p_idpoblacio AND ROWNUM = 1;
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            RAISE err_no_idpoblac;
      END;

      -- Existeix la poblacio
      BEGIN
         v_count := NULL;

         SELECT 1
           INTO v_count
           FROM poblacio
          WHERE nompoblacio = p_nompoblacio AND ROWNUM = 1;

         RAISE err_nompoblacio;
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            NULL;
      END;

      -- Existeix Id a pais
      BEGIN
         v_count := NULL;

         SELECT 1
           INTO v_count
           FROM pais
          WHERE idpais = p_idpais AND ROWNUM = 1;
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            RAISE err_no_pais;
      END;

      -- Si compleix els requisits, se fa la modificaci
      UPDATE POBLACIO
         SET NOMPOBLACIO = NVL (p_nompoblacio, NOMPOBLACIO), IDPAIS = NVL (p_idpais, IDPAIS)
       WHERE IDPOBLACIO = p_idpoblacio;

      RSP := 'OK: modificaci de poblaci realitzada correctament.';

      INSERT INTO LOGS (NOMPROC,
                        LOGDATE,
                        PARIN,
                        PAROUT)
           VALUES ('pkg_abm.prc_modif_poblacio',
                   SYSDATE,
                   v_parin,
                   'RSP = ' || RSP);

      COMMIT;
   EXCEPTION
      -- Si no se pot fer l'insert, se genera un registre de error a la taula LOGS
      WHEN err_null
      THEN
         RSP := 'ERROR: hi ha camps obligatoris sense informar';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_modif_poblacio',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN err_no_idpoblac
      THEN
         RSP := 'ERROR: La poblaci no existeix';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_modif_poblacio',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN err_nompoblacio
      THEN
         RSP := 'ERROR: El nom de la poblaci ja existeix';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_modif_poblacio',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN err_no_pais
      THEN
         RSP := 'ERROR: El pas no existeix a la base de dades';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_modif_poblacio',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN OTHERS
      THEN
         err_num := SQLCODE;
         err_msg := SUBSTR (SQLERRM, 1, 200);

         RSP := 'Error no tractat: ' || TO_CHAR (err_num) || ': ' || err_msg;

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_alta_poblacio',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
   END prc_modif_poblacio;

   /************************************* Localitat *******************************************/

   PROCEDURE prc_alta_localitat (p_nomlocalitat   IN     localitat.nomlocalitat%TYPE,
                                 p_idpoblacio     IN     localitat.idpoblacio%TYPE,
                                 p_codpostal      IN     localitat.codpostal%TYPE,
                                 RSP                 OUT VARCHAR2)
   IS
      /*******************************************************************************************
          NOM:        prc_alta_localitat
          PROPSIT:   Dona d'alta una localitat
          REQUISITS:  - Han d'estar informats tots els camps obligatoris (NOT NULL)
                      - El registre no ha d'existir (codi postal)
                      - L'Id de poblaci ha d'existir a la base de dades
          PARMETRES:
              IN:   Obligatoris:  p_idpoblacio: Identificador de la poblaci
                                  p_nomlocalitat: Nom de la localitat
                                  p_codpostal: Codi postal de la localitat
              OUT:  RSP
          PRE-CONDICI:  No existeix la localitat
          POST-CONDICI: Se dona d'alta la localitat
          RETORNA:
              tot ha anat be:
                  RSP = 'OK: alta de localitat realitzada correctament. Id = NumId'
              missatge de errors:
                  'ERROR: hi ha camps obligatoris sense informar'
                  'ERROR: La localitat ja existeix'
                  'ERROR: La poblaci no existeix a la base de dades'
                  'ERROR: Error no tractat'
      *******************************************************************************************/
      err_null          EXCEPTION;
      err_codpostal     EXCEPTION;
      err_no_poblacio   EXCEPTION;

      v_count           VARCHAR2 (1);
      v_parin           VARCHAR2 (300);
      v_seq             NUMBER;
      err_num           NUMBER;
      err_msg           VARCHAR2 (100);
   BEGIN
      v_parin :=
            'p_nomlocalitat: '
         || p_nomlocalitat
         || ', p_idpoblacio: '
         || p_idpoblacio
         || ', p_codpostal: '
         || p_codpostal;

      -- Verificaci de requisits
      -- Han d'estar informats tots els camps obligatoris (NOT NULL)
      IF (   TRIM (p_idpoblacio) IS NULL
          OR TRIM (p_nomlocalitat) IS NULL
          OR TRIM (p_codpostal) IS NULL)
      THEN
         RAISE err_null;
      END IF;

      -- Existeix la localitat
      BEGIN
         v_count := NULL;

         SELECT 1
           INTO v_count
           FROM localitat
          WHERE codpostal = p_codpostal AND ROWNUM = 1;

         RAISE err_codpostal;
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            NULL;
      END;

      -- Existeix Id a poblaci
      BEGIN
         v_count := NULL;

         SELECT 1
           INTO v_count
           FROM poblacio
          WHERE idpoblacio = p_idpoblacio AND ROWNUM = 1;
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            RAISE err_no_poblacio;
      END;

      -- Si compleix els requisits, se fa la inserci
      INSERT INTO LOCALITAT (IDPOBLACIO, NOMLOCALITAT, CODPOSTAL)
           VALUES (p_idpoblacio, p_nomlocalitat, p_codpostal);

      SELECT seq_LOCALITAT.CURRVAL INTO v_seq FROM DUAL;

      RSP := 'OK: alta de localitat realitzada correctament. Id = ' || v_seq;

      INSERT INTO LOGS (NOMPROC,
                        LOGDATE,
                        PARIN,
                        PAROUT)
           VALUES ('pkg_abm.prc_alta_localitat',
                   SYSDATE,
                   v_parin,
                   'RSP = ' || RSP);

      COMMIT;
   EXCEPTION
      -- Si no se pot fer l'insert, se genera un registre de error a la taula LOGS
      WHEN err_null
      THEN
         RSP := 'ERROR: hi ha camps obligatoris sense informar';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_alta_localitat',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN err_codpostal
      THEN
         RSP := 'ERROR: La localitat ja existeix';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_alta_localitat',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN err_no_poblacio
      THEN
         RSP := 'ERROR: La poblaci no existeix a la base de dades';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_alta_localitat',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN OTHERS
      THEN
         err_num := SQLCODE;
         err_msg := SUBSTR (SQLERRM, 1, 200);

         RSP := 'Error no tractat: ' || TO_CHAR (err_num) || ': ' || err_msg;

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_alta_localitat',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
   END prc_alta_localitat;

   PROCEDURE prc_borra_localitat (p_idlocalitat IN localitat.idlocalitat%TYPE, RSP OUT VARCHAR2)
   IS
      /*******************************************************************************************
          NOM:        prc_borra_localitat
          PROPSIT:   Borra una localitat
          REQUISITS:  - S'ha d'informar l'Id de la localitat
                      - El registre ha d'existir a la base de dades
                      - El registre no ha d'estar lligat a cap taula externa
                        (central_produccio, central_distribucio, comptador, client)
          PARMETRES:
              IN:   Obligatoris:  p_idlocalitat: Identificador de la localitat
              OUT:  RSP
          PRE-CONDICI:  Existeix la localitat que se vol borrar
          POST-CONDICI: se borra la localitat
          RETORNA:
              tot ha anat be:
                  RSP = 'OK: delete de localitat realitzat correctament'
              missatge de errors:
                  'ERROR: hi ha camps obligatoris sense informar'
                  'ERROR: La localitat no existeix'
                  'ERROR: Hi ha centrals de producci lligades a aquesta localitat'
                  'ERROR: Hi ha centrals de distribuci lligades a aquesta localitat'
                  'ERROR: Hi ha comptadors lligats a aquesta localitat'
                  'ERROR: Hi ha clients lligats a aquesta localitat'
                  'ERROR: Error no tractat'
      *******************************************************************************************/
      err_null                EXCEPTION;
      err_no_idlocalitat      EXCEPTION;
      err_idloc_centralprod   EXCEPTION;
      err_idloc_centraldis    EXCEPTION;
      err_idloc_comptador     EXCEPTION;
      err_idloc_client        EXCEPTION;

      v_count                 VARCHAR2 (1);
      v_parin                 VARCHAR2 (300);
      err_num                 NUMBER;
      err_msg                 VARCHAR2 (100);
   BEGIN
      v_parin := 'p_idlocalitat: ' || p_idlocalitat;

      -- Verificaci de requisits
      IF (TRIM (p_idlocalitat) IS NULL)
      THEN
         RAISE err_null;
      END IF;

      -- Existeix Id a localitat
      BEGIN
         v_count := NULL;

         SELECT 1
           INTO v_count
           FROM localitat
          WHERE idlocalitat = p_idlocalitat AND ROWNUM = 1;
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            RAISE err_no_idlocalitat;
      END;

      -- El registre no est lligat a central_produccio
      BEGIN
         v_count := NULL;

         SELECT 1
           INTO v_count
           FROM central_produccio
          WHERE idlocalitat = p_idlocalitat AND ROWNUM = 1;

         RAISE err_idloc_centralprod;
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            NULL;
      END;

      -- El registre no est lligat a central_distribucio
      BEGIN
         v_count := NULL;

         SELECT 1
           INTO v_count
           FROM central_distribucio
          WHERE idlocalitat = p_idlocalitat AND ROWNUM = 1;

         RAISE err_idloc_centraldis;
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            NULL;
      END;

      -- El registre no est lligat a comptador
      BEGIN
         v_count := NULL;

         SELECT 1
           INTO v_count
           FROM comptador
          WHERE idlocalitat = p_idlocalitat AND ROWNUM = 1;

         RAISE err_idloc_comptador;
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            NULL;
      END;

      -- El registre no est lligat a client
      BEGIN
         v_count := NULL;

         SELECT 1
           INTO v_count
           FROM client
          WHERE idlocalitat = p_idlocalitat AND ROWNUM = 1;

         RAISE err_idloc_client;
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            NULL;
      END;

      -- Si compleix els requisits, se fa un delete de la localitat
      DELETE FROM LOCALITAT
            WHERE IDLOCALITAT = p_idlocalitat;

      RSP := 'OK: delete de localitat realitzat correctament.';

      INSERT INTO LOGS (NOMPROC,
                        LOGDATE,
                        PARIN,
                        PAROUT)
           VALUES ('pkg_abm.prc_borra_localitat',
                   SYSDATE,
                   v_parin,
                   'RSP = ' || RSP);

      COMMIT;
   EXCEPTION
      -- Si no se pot fer l'insert, se genera un registre de error a la taula LOGS
      WHEN err_null
      THEN
         RSP := 'ERROR: hi ha camps obligatoris sense informar';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_borra_localitat',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN err_no_idlocalitat
      THEN
         RSP := 'ERROR: La localitat no existeix';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_borra_localitat',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN err_idloc_centralprod
      THEN
         RSP := 'ERROR: Hi ha centrals de producci lligades a aquesta localitat';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_borra_localitat',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN err_idloc_centraldis
      THEN
         RSP := 'ERROR: Hi ha centrals de distribuci lligades a aquesta localitat';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_borra_localitat',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN err_idloc_comptador
      THEN
         RSP := 'ERROR: Hi ha comptadors lligats a aquesta localitat';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_borra_localitat',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN err_idloc_client
      THEN
         RSP := 'ERROR: Hi ha clients lligats a aquesta localitat';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_borra_localitat',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN OTHERS
      THEN
         err_num := SQLCODE;
         err_msg := SUBSTR (SQLERRM, 1, 200);

         RSP := 'Error no tractat: ' || TO_CHAR (err_num) || ': ' || err_msg;

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_borra_localitat',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
   END prc_borra_localitat;

   PROCEDURE prc_modif_localitat (p_idlocalitat    IN     localitat.idlocalitat%TYPE,
                                  p_idpoblacio     IN     localitat.idpoblacio%TYPE,
                                  p_nomlocalitat   IN     localitat.nomlocalitat%TYPE,
                                  p_codpostal      IN     localitat.codpostal%TYPE,
                                  RSP                 OUT VARCHAR2)
   IS
      /*******************************************************************************************
          NOM:        prc_modif_localitat
          PROPSIT:   Modifica una localitat
          REQUISITS:  - S'ha d'informar l'Id de la localitat
                      - L'Id de la poblaci ha d'existir a la base de dades
          PARMETRES:
              IN:   Obligatoris:  p_idlocalitat: Identificador de la localitat
                    Opcionals:    p_idpoblacio: Identificador de la poblaci
                                  p_nomlocalitat: Nom de la localitat
                                  p_codpostal: Codi postal de la localitat
              OUT:  RSP
          PRE-CONDICI:  Existeix la localitat que se vol modificar
          POST-CONDICI: Se modifica la localitat
          RETORNA:
              tot ha anat be:
                  RSP = 'OK: modificaci de localitat realitzada correctament'
              missatge de errors:
                  'ERROR: hi ha camps obligatoris sense informar'
                  'ERROR: La localitat no existeix'
                  'ERROR: El codi postal ja existeix'
                  'ERROR: La poblaci no existeix a la base de dades'
                  'ERROR: Error no tractat'
      *******************************************************************************************/
      err_null           EXCEPTION;
      err_codpostal      EXCEPTION;
      err_no_localitat   EXCEPTION;
      err_no_poblacio    EXCEPTION;

      v_count            VARCHAR2 (1);
      v_parin            VARCHAR2 (300);
      err_num            NUMBER;
      err_msg            VARCHAR2 (100);
   BEGIN
      v_parin :=
            'p_idlocalitat: '
         || p_idlocalitat
         || ', p_idpoblacio: '
         || p_idpoblacio
         || ', p_nomlocalitat: '
         || p_nomlocalitat
         || ', p_codpostal: '
         || p_codpostal;

      -- Verificaci de requisits
      -- Han d'estar informats tots els camps obligatoris (NOT NULL)
      IF (TRIM (p_idlocalitat) IS NULL)
      THEN
         RAISE err_null;
      END IF;

      -- Existeix la localitat
      BEGIN
         v_count := NULL;

         SELECT 1
           INTO v_count
           FROM localitat
          WHERE idlocalitat = p_idlocalitat AND ROWNUM = 1;
         
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
           RAISE err_no_localitat;
      END;

      -- Existeix el codi postal
      BEGIN
         v_count := NULL;

         SELECT 1
           INTO v_count
           FROM localitat
          WHERE codpostal = p_codpostal AND ROWNUM = 1;

         RAISE err_codpostal;
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            NULL;
      END;

      -- Existeix Id a poblaci
      BEGIN
         v_count := NULL;

         SELECT 1
           INTO v_count
           FROM poblacio
          WHERE idpoblacio = p_idpoblacio AND ROWNUM = 1;
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            RAISE err_no_poblacio;
      END;

      -- Si compleix els requisits, se fa la modificaci
      UPDATE LOCALITAT
         SET IDPOBLACIO = NVL (p_idpoblacio, IDPOBLACIO),
             NOMLOCALITAT = NVL (p_nomlocalitat, NOMLOCALITAT),
             CODPOSTAL = NVL (p_codpostal, CODPOSTAL)
       WHERE IDLOCALITAT = p_idlocalitat;

      RSP := 'OK: modificaci de localitat realitzada correctament';

      INSERT INTO LOGS (NOMPROC,
                        LOGDATE,
                        PARIN,
                        PAROUT)
           VALUES ('pkg_abm.prc_modif_localitat',
                   SYSDATE,
                   v_parin,
                   'RSP = ' || RSP);

      COMMIT;
   EXCEPTION
      -- Si no se pot fer l'insert, se genera un registre de error a la taula LOGS
      WHEN err_null
      THEN
         RSP := 'ERROR: hi ha camps obligatoris sense informar';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_modif_localitat',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN err_codpostal
      THEN
         RSP := 'ERROR: El codi postal ja existeix';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_modif_localitat',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN err_no_localitat
      THEN
         RSP := 'ERROR: La localitat no existeix';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_modif_localitat',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN err_no_poblacio
      THEN
         RSP := 'ERROR: La poblaci no existeix a la base de dades';

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_modif_localitat',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
      WHEN OTHERS
      THEN
         err_num := SQLCODE;
         err_msg := SUBSTR (SQLERRM, 1, 200);

         RSP := 'Error no tractat: ' || TO_CHAR (err_num) || ': ' || err_msg;

         INSERT INTO LOGS (NOMPROC,
                           LOGDATE,
                           PARIN,
                           PAROUT)
              VALUES ('pkg_abm.prc_modif_localitat',
                      SYSDATE,
                      v_parin,
                      'RSP = ' || RSP);

         COMMIT;
   END prc_modif_localitat;
END pkg_abm;
/

spool off;